/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import java.util.logging.*;
import cgl.globalmmcs.webserver.sessionmgt.*;
import cgl.globalmmcs.webserver.admin.*;
import cgl.globalmmcs.webserver.admin.commMgt.*;
import cgl.globalmmcs.webserver.admin.meetingMgt.*;
import cgl.globalmmcs.webserver.admin.userMgt.*;
import cgl.globalmmcs.webserver.admin.roomMgt.*;

import cgl.globalmmcs.webserver.user.RegisterComm;
import cgl.globalmmcs.webserver.user.LogIn;
import cgl.globalmmcs.webserver.user.LogOut;


public class AdminServlet extends HttpServlet {
	private static final String CONTENT_TYPE = "text/html";
	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	private HashMap commands;
	private String Error = "error.jsp";
	private String jspdir = "/";

	//Initialize global variables
	public void init (ServletConfig config) throws ServletException {
		InputStream is = null;
		try {
			is = new FileInputStream ("webserver.log.properties");
		} catch (FileNotFoundException ex) {
			ex.printStackTrace ();
		}

		LogManager lm = LogManager.getLogManager ();
		try {
			lm.readConfiguration (is);
		} catch (Exception ex) {
			ex.printStackTrace ();
		}

		super.init (config);
		initCommands ();
		// initilize the XGSP Configuration
		String H323GW = getInitParameter ("H323GW_ADDR");
		String SIPGW = getInitParameter ("SIPGW_ADDR");
		String Sv_Addr = getInitParameter ("SESSIONSV_ADDR");
		String Helix_Addr = getInitParameter ("HelixSV_ADDR");
		String Helix_Port = getInitParameter ("HelixSV_PORT");
		String RealGW = getInitParameter ("RealGW_ADDR");
		String DB_URL = getInitParameter ("DATABASE_URL");
		String DB_Path = getInitParameter ("DATABASE_PATH");
		String NB_Addr = getInitParameter ("NARADAB_ADDR");
		String NB_Port = getInitParameter ("NARADAB_PORT");
		XGSPConfiguration xgsp_conf = XGSPConfiguration.getInstance ();

		logger.info ("H323GW :"+H323GW);
		logger.info ("SIPGW :"+SIPGW);
		logger.info ("Sv_Addr :"+Sv_Addr);
		logger.info ("DB_Path :"+DB_Path);
		logger.info ("NB_Addr :"+NB_Addr);
		logger.info ("NB_Port :"+NB_Port);

		xgsp_conf.setH323GW (H323GW);
		xgsp_conf.setSIPGW (SIPGW);
		xgsp_conf.setSessionSv (Sv_Addr);
		xgsp_conf.setHelixSvr (Helix_Addr);
		xgsp_conf.setHelixPort (Helix_Port);
		xgsp_conf.setRealGW (RealGW);
		xgsp_conf.setDataBaseURL (DB_URL);
		xgsp_conf.setDataBaseName (DB_Path);
		xgsp_conf.setNBAddr (NB_Addr);
		xgsp_conf.setNBPort (NB_Port);
		// start the hearbeat monitor
		HeartBeatProcess heartbeat_mon = new HeartBeatProcess ();
		heartbeat_mon.start ();
		// start ActiveSessionDemon here:
		XGSPAVSessionManager manager = XGSPAVSessionManager.getInstance ();
		manager.StartSessionDemo ();
	}


	// Main Control Routine
	public void service (HttpServletRequest req, HttpServletResponse res) throws
			 ServletException, IOException {
		logger.info ("AdminServlet ... service ... enter");
		//
		String cmd_str = null;
		UserBean user;
		HttpSession session;

		//
		req.setAttribute ("servletName", "admin");

		//
		session = req.getSession (true);
		user = (UserBean) session.getAttribute ("userLogged");

		//
		cmd_str = req.getParameter ("cmd");

		//
		if (user==null) {
			logger.info ("user null");

			if (cmd_str!=null) {
				if (!((cmd_str.equalsIgnoreCase ("do-login"))||
					 (cmd_str.equalsIgnoreCase ("do-register"))||
					 (cmd_str.equalsIgnoreCase ("register"))
					 )) {
					cmd_str = "login";
				}
			}
		}

		if (user!=null) {
			logger.info ("user : "+user);
		}

		String next = null;
		try {
			Command cmd = lookupCommand (cmd_str);
			next = cmd.execute (req);
			logger.info ("cmd_str : "+cmd_str);
			logger.info ("next    : "+next);

		} catch (Exception e) {
			logger.severe (e.getMessage ());
			logger.severe (e.toString ());
			logger.severe ("cmd_str : "+cmd_str);

			req.setAttribute ("javax.servlet.jsp.jspException", e);
			next = Error;
			logger.severe ("next    : "+next);
		}

		RequestDispatcher rd;
		rd = getServletContext ().getRequestDispatcher (jspdir+next);
		logger.info ("next    : "+next);

		logger.info ("jspdir+next    : "+jspdir+next);
		rd.forward (req, res);
		logger.info ("AdminServlet ... service ... exit");
		/*
		    String next;
		    String cmdName = "unknown";
		    try {
		  cmdName = req.getParameter("cmd");
		  Command cmd = lookupCommand(cmdName);
		  next = cmd.execute(req);
		    }
		    catch (CommandException e) {
		  req.setAttribute("javax.servlet.jsp.jspException", e);
		  next = Error;
		  logger.info("can't find the command " + cmdName);
		    }
		    catch (UnknownCommandException e) {
		  req.setAttribute("javax.servlet.jsp.jspException", e);
		  next = Error;
		  logger.info("can't find the command " + cmdName);
		    }
		    RequestDispatcher rd;
		    rd = getServletContext().getRequestDispatcher(jspdir + next);
		    rd.forward(req, res);
		 */
	}


	/*
	  private Command lookupCommand(String cmd) throws UnknownCommandException {
	    if (cmd == null) {
	  //cmd = "main-menu";
	  //cmd = "manage-community";
	  //cmd = "manage-room";
	  cmd = "admin-main";
	    }
	    if (commands.containsKey(cmd.toLowerCase())) {
	  return (Command) commands.get(cmd.toLowerCase());
	    }
	    else {
	  throw new UnknownCommandException("Invalid Command Identifier");
	    }
	  }
	 */
	private Command lookupCommand (String cmd) throws CommandException {
		logger.info ("AdminServlet ... lookupCommand ... enter");
		Command c;

		try {
			if (commands==null) {
				logger.severe ("commands is null !!!!!!");
			}

			if (cmd==null) {
				logger.info ("cmd is null");
				logger.info ("cmd is NOW ACCEPTED TO BE: admin-main");

				c = (Command) commands.get ("admin-main");

				logger.info ("c.getClass().getName() : "+c.getClass ().getName ());
				logger.info ("AdminServlet ... lookupCommand ... exit");
				return c;
			}

			if (commands.containsKey (cmd.toLowerCase ())) {
				logger.info ("cmd is NOT null");
				logger.info ("cmd is : "+cmd);

				c = (Command) commands.get (cmd.toLowerCase ());

				logger.info ("c.getClass().getName() : "+c.getClass ().getName ());
				logger.info ("AdminServlet ... lookupCommand ... exit");
				return c;
			}

			logger.info ("throw new CommandException (\"Invalid Command Identifier\")");
			logger.info (
					 "AdminServlet ... lookupCommand ... exit with CommandException");
		} catch (Exception e) {
			e.printStackTrace ();
		}

		throw new CommandException ("Invalid Command Identifier");
	}


	// InitCommands
	private void initCommands () {
		logger.info ("AdminServlet ... initCommands ... enter");

		commands = new HashMap ();
		// Login should be the first command
		// login & logout
		commands.put ("login", new NullCommand ("login.jsp")); // login: 1
		commands.put ("do-login", new LogIn ("welcome.jsp")); // login: 2
		commands.put ("logout", new LogOut ("login.jsp")); // logout ...
		commands.put ("do-register", new RegisterComm ("login.jsp")); // register:2 ...
		commands.put ("register", new NullCommand ("register.jsp")); // register:1 ...

		//commands.put("main-menu", new NullCommand("login.jsp"));
		commands.put ("admin-main", new NullCommand ("admin.jsp"));
		commands.put ("xgsp_conf-jsp", new NullCommand ("xgsp_conf.jsp"));
		commands.put ("sessdemonctrl-jsp", new NullCommand ("SessDemonCtrl.jsp"));
		commands.put ("system_mon-jsp", new NullCommand ("system_mon.jsp"));

		// System Configuration Commands
		commands.put ("conf-xgsp", new XgspconfExec ("xgsp_conf_success.jsp"));

		commands.put ("edit-user", new getUserASpecificUser ("edituserAdmin.jsp")); // update:1 .....   //I added

		// User Management Commands
		//commands.put("login", new NullCommand("login.jsp"));
		//commands.put("register", new NullCommand("register.jsp"));
		//commands.put("do-login", new LogInCommand("welcome.jsp"));
		//commands.put("do-register",new RegisterCommand("welcome.jsp"));
		commands.put ("get-user", new getUserASpecificUser ("userinfo.jsp"));

		commands.put ("manage-user", new GetAllUser ("manageuser.jsp"));
		// commands.put("get-user", new getUser("userinfo.jsp"));
		commands.put ("add-user", new AddUser ("manageuser.jsp")); //Harun added
		commands.put ("delete-user", new deleteUser ("manageuser.jsp")); //Harun added
		// edit/update user
		// ?? commands.put ("edit-user", new getUserASpecificUser ("edituserAdmin.jsp")); // update:1 .....   //Harun added
		commands.put ("update-user", new editUser ("manageuser.jsp")); // update:2 .....   //Harun added

		// Community Management Commands
		commands.put ("manage-community", new GetAllComm ("managecomm.jsp"));
		commands.put ("get-community", new getCommunity ("communityinfo.jsp"));
		commands.put ("add-community", new AddCommunity ("managecomm.jsp"));
		commands.put ("delete-community", new deleteCommunity ("managecomm.jsp"));
		commands.put ("edit-community", new getCommunity ("editcommunity.jsp"));
		commands.put ("update-community", new editCommunity ("managecomm.jsp"));

		// Room Management Commands
		commands.put ("manage-room", new GetAllRoom ("manageroom.jsp"));
		commands.put ("get-room", new getRoom ("roominfo.jsp"));
		commands.put ("add-room", new AddRoom ("manageroom.jsp"));
		commands.put ("edit-room", new getRoom ("editroom.jsp"));
		commands.put ("update-room", new editRoom ("manageroom.jsp"));
		commands.put ("delete-room", new deleteRoom ("manageroom.jsp"));
		commands.put ("sync-room", new syncRoom ("manageroom.jsp"));
		commands.put ("check-sync", new checkSyncRoom ("manageroom.jsp"));
		// Session Management Commands
		/**
		 * GetAllSession --> managersession.jsp
		 * managersession.jsp --> AddSession.java
		 * "cmd=get-session" --> sessioninfo.jsp --> [ (modify)edit-session, (delete)delete-session ]
		 * add-session: AddSession.java --> managesession.jsp
		 * edit-session :  getSession.java --> editSession.jsp --> editSession.java --> managesession.jsp
		 * delete-session: deleteSession.java --> managesession.jsp
		 */
		commands.put ("manage-session", new GetAllSession ("managesessionAdmin.jsp"));
		commands.put ("get-session", new getSession ("sessioninfoAdmin.jsp"));
		commands.put ("find-session", new findSessions ("managesessionAdmin.jsp"));
		commands.put ("delete-session", new deleteSession ("managesessionAdmin.jsp"));
		commands.put ("add-session", new AddSession ("managesessionAdmin.jsp"));
		commands.put ("edit-session", new getSession ("editSession.jsp"));
		commands.put ("update-session", new editSession ("managesessionAdmin.jsp"));
		commands.put ("sync-session", new syncSession ("managesessionAdmin.jsp"));
		commands.put ("add-session-empty", new NullCommand ("addsession.jsp"));
		// Session Monitor Commands
		// these commands are mostly about video mixer management
		/** ShowAllActiveSession --> SelActiveSess.jsp --> ShowAllMixer -->
		 *  SelVideoMixer.jsp --> [ Modify, Delete, Create, Back ]
		 *  Modify --> ModVideoMixer.java --> SelVideoMixer
		 *  Delete --> DelMixer.java --> SelVideoMixer
		 *  Add --> AddMixer.java --> SelVideoMixer
		 */
		commands.put ("monitor-session",
				    new ShowAllActiveSession ("SelActiveSess.jsp"));
		commands.put ("mon-session", new MonActiveSess ("MonActiveSess.jsp"));
		commands.put ("link-session", new LinkSession ("LinkSuccess.jsp"));

		// ActiveSession Scheduler Demon Commands
		commands.put ("start-demon", new StartDemon ("SessDemonCtrl.jsp"));
		commands.put ("stop-demon", new StopDemon ("SessDemonCtrl.jsp"));
		commands.put ("start-manager", new StartManager ("SessDemonCtrl.jsp"));

		commands.put ("manage-mixer", new ShowAllMixer ("SelVideoMixer.jsp"));

		commands.put ("add-video-mixer", new AddVideoMixer ("SelVideoMixer.jsp"));
		commands.put ("edit-video-mixer", new ModVideoMixer ("SelVideoMixer.jsp"));
		commands.put ("delete-video-mixer", new DelVideoMixer ("SelVideoMixer.jsp"));

		// RealStreaming Gateway Admin Commands
				 /** Since the basic functions is implemented in an Admin Applet, we just pop a web page as a container for it **/
		commands.put ("manage-realgw", new StartRealAdmin ("StreamingAdmin.jsp"));
		logger.info ("AdminServlet ... initCommands ... exit");
	}

}