/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

import java.sql.*;
import java.io.*;
import java.util.*;


public class CommunityRepository {
	private static CommunityRepository instance;
	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	private Connection connection;
	private PreparedStatement getStmt;
	private PreparedStatement putStmt;
	private PreparedStatement getAllStmt;
	private PreparedStatement remStmt;
	private PreparedStatement updStmt;

	// Single Object Creation
	public static CommunityRepository getInstance () throws
			 CommRepositoryException {
		if (instance==null) {
			instance = new CommunityRepository ();
		}
		return instance;
	}


	private CommunityRepository () throws CommRepositoryException {
		String get = "SELECT * FROM communities WHERE ID=?";
		String put = "INSERT INTO communities VALUES ( ?,?,?,? )";
		String all = "SELECT * FROM communities ORDER BY ID";
		String rem = "DELETE FROM communities WHERE ID=?";
		String upd = "UPDATE communities SET NAME=?, TYPE=?, ADDR=? WHERE ID=?";
		// get the URL & file name of the database
		XGSPConfiguration xgsp_conf = XGSPConfiguration.getInstance ();

		try {
			DBConnection db = new DBConnection ();
			connection = db.getConnection ();

			getStmt = connection.prepareStatement (get);
			putStmt = connection.prepareStatement (put);
			getAllStmt = connection.prepareStatement (all);
			remStmt = connection.prepareStatement (rem);
			updStmt = connection.prepareStatement (upd);
		} catch (ClassNotFoundException e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
//         logger.severe ("Exception : "+e.getMessage ());
			//       logger.severe ("Exception : "+e.toString ());

			throw new CommRepositoryException ("No Driver Available!");
		} catch (SQLException e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			throw new CommRepositoryException (e.getMessage ());
		} catch (Exception e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			e.printStackTrace ();
		}

	}


	// Methods for CommunityBean Object
	private CommunityBean makeCommunity (ResultSet results) throws
			 CommRepositoryException {
		try {
			CommunityBean community = new CommunityBean ();
			community.setCommunityid (results.getString ("ID"));
			community.setCommunityname (results.getString ("NAME"));
			community.setCommunitytype (results.getString ("TYPE"));
			community.setCommunityaddr (results.getString ("ADDR"));
			return community;
		} catch (SQLException e) {
			throw new CommRepositoryException (e.getMessage ());
		}
	}


	// Query CommunityBean
	public CommunityBean getCommunity (String CommunityId) throws
			 UnknownCommException, CommRepositoryException {
		try {
			ResultSet results;
			synchronized (getStmt) { //query community table
				getStmt.clearParameters ();
				getStmt.setString (1, CommunityId);
				results = getStmt.executeQuery ();
			}
			if (results.next ()) {
				return makeCommunity (results);
			} else {
				throw new UnknownCommException ("could not find Community"+CommunityId);
			}
		} catch (SQLException e) {
			throw new CommRepositoryException (e.getMessage ());
		}
	}


	// Get all Coomunity records
	public CommunityBean[] getComms () throws CommRepositoryException {
		try {
			ResultSet results;
			Collection comms = new ArrayList ();
			synchronized (getAllStmt) {
				results = getAllStmt.executeQuery ();
			}
			CommunityBean comm;
			while (results.next ()) {
				comms.add (makeCommunity (results));
			}
			return (CommunityBean[]) comms.toArray (new CommunityBean[0]);
		} catch (SQLException e) {
			throw new CommRepositoryException (e.getMessage ());
		}
	}


	// Put CommunityBean into the Community Table
	public void put (CommunityBean community) throws
			 CommRepositoryException {
		try {
			synchronized (putStmt) {
				putStmt.clearParameters ();
				putStmt.setString (1, community.getCommunityid ());
				putStmt.setString (2, community.getCommunityname ());
				putStmt.setString (3, community.getCommunitytype ());
				putStmt.setString (4, community.getCommunityaddr ());
				putStmt.executeUpdate ();
			}
		} catch (SQLException e) {
			throw new CommRepositoryException (e.getMessage ());
		}
	}


	// Delete a Community Record
	public void removeCommunity (String communityid) throws
			 CommRepositoryException {
		try {
			synchronized (remStmt) {
				remStmt.clearParameters ();
				remStmt.setString (1, communityid);
				int rowsChanged = remStmt.executeUpdate ();
				if (rowsChanged<1) {
					throw new UnknownCommException ("Can't delete Community# "+
							 communityid);
				}
			}
		} catch (SQLException e) {
			throw new CommRepositoryException (e.getMessage ());
		}
	}


	// Update the old community record
	public void update (CommunityBean comm) throws
			 UnknownCommException, CommRepositoryException {
		try {
			synchronized (updStmt) {
				updStmt.clearParameters ();
				updStmt.setString (4, comm.getCommunityid ());
				updStmt.setString (1, comm.getCommunityname ());
				updStmt.setString (2, comm.getCommunitytype ());
				updStmt.setString (3, comm.getCommunityaddr ());
				int rowsChanged = updStmt.executeUpdate ();
				if (rowsChanged<1) {
					throw new UnknownCommException ("Could not find Comm# "+
							 comm.getCommunityid ());
				}
			}
		} catch (SQLException e) {
			throw new CommRepositoryException (e.getMessage ());
		}
	}

}
