/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

/**
 *
 * <p>Title: </p>
 * <p>Description: It implements a HeartBeatProcess for monitor other servers in GlobalMMCS </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Community Grids Lab, Indiana University</p>
 * @author Wenjun Wu, Harun Altay
 * @version 1.0
 */

import java.util.*;
import java.lang.*;
import java.net.*;
import java.io.*;


/**
 * @deprecated
 * because we use UDP multicast to do it. In the next step, it has to be moved to NaradaBroker
 * It only defines the servers based on the old architecture
 * @deprecated totally : 6 Servers
 * H323Gateway, H323Gatekeeper, SessionServer, AudioServer, VideoServer, SnapShotServer & WebServer
 * (0,1,2,3,4,5)
 * Right now we have servers including:
 * H323Gateway, H323Gatekeeper, SessionServer, VideoMixer, AudioMixer, ImageGrabber, RealGateway
 */

class ServStateTask extends TimerTask {
	private String ServerName;
	public ServStateTask (String server_name) {
		ServerName = server_name;
	}


	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	public void run () {
		// reset the soft state to not alive
		XGSPConfiguration xgspconf = XGSPConfiguration.getInstance ();
		xgspconf.setServerState (ServerName, false);
	}

}


class HeartBeatSender extends TimerTask {
	private MulticastSocket sender_socket;
	private InetAddress sender_grp;
	private int seqnum;
	public HeartBeatSender (MulticastSocket s, InetAddress grp) {
		sender_socket = s;
		sender_grp = grp;
		seqnum = 0;
	}


	public void run () {
		String tmp = (new Integer (seqnum)).toString ()+" WebServer ";
		byte[] msg = tmp.getBytes ();
		try {
			DatagramPacket heart_msg = new DatagramPacket (msg, msg.length, sender_grp,
					 5000);
			sender_socket.send (heart_msg);
			seqnum++;
		} catch (IOException ie) {
			System.err.println (ie);
		}
	}
}


public class HeartBeatProcess extends Thread {
	private XGSPConfiguration xgspconf;
	private Timer[] softstate_timer = new Timer[7];
	private Timer sender_timer = new Timer ();

	public HeartBeatProcess () {
		xgspconf = XGSPConfiguration.getInstance ();
		for (int i = 0; i<7; i++) {
			softstate_timer[i] = new Timer ();
		}
	}


	public void run () {
		String seqnum, server_name = "";
		try {
			// listening on the 224.1.1.44/5000
			InetAddress group = InetAddress.getByName ("224.1.1.44");
			MulticastSocket s = new MulticastSocket (4000);
			s.joinGroup (group);

			//DatagramPacket hi = new DatagramPacket(msg, msg.length, group, 6789);
			//s.send(hi);
			// Schedule heart beating sender
			sender_timer.schedule (new HeartBeatSender (s, group), 0, 1000);
			// create timers for each state of the server

			while (true) {
				// get every Heartbeat Message from the address
				byte[] buf = new byte[100];
				DatagramPacket recv = new DatagramPacket (buf, buf.length);
				s.receive (recv);
				// set this server alive
				String message = new String (recv.getData ());
				// retrieve the HeartBeat Message: < seqnum, ServerName >
				java.util.StringTokenizer st = new java.util.StringTokenizer (message,
						 " ");
				int cnt = 0;
				while (st.hasMoreTokens ()) {
					String tmp = st.nextToken ();
					if (cnt==0) {
						seqnum = tmp;
					}
					if (cnt==1) {
						server_name = tmp;
					}
					cnt++;
				}
				//System.out.println(message);
				// cancel the old Timer, if the state of this server has been false, then
				// it means that the Timer has been fired and no neccessary to cancel
				int server_index = xgspconf.getServerIndex (server_name);
				if (xgspconf.getServerState (server_name)) {
					softstate_timer[server_index].cancel ();
					softstate_timer[server_index] = new Timer ();
				}
				xgspconf.setServerState (server_name, true);
				// reschedule the timer task
				softstate_timer[server_index].schedule (new ServStateTask (server_name),
						 1000*5);
			}
		} catch (IOException ie) {
			System.err.println (ie);
		}
		// OK, I'm done talking - leave the group...
		//s.leaveGroup(group);
	}

}