/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

import java.sql.*;
import java.io.*;
import java.util.*;


// Storage Module
/** Methods
 ** RoomRepository()
 ** makeRoom(html-result), getRoom(Room-id), put(Room-bean)
 ** roomid communityid roomname roomaddr roomtype roomstatus
 **/
public class RoomRepository {
	private static RoomRepository instance;
	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	private Connection connection;
	private PreparedStatement getStmt;
	private PreparedStatement putStmt;
	private PreparedStatement remStmt;
	private PreparedStatement updStmt;
	private PreparedStatement getAllStmt;

	public static RoomRepository getInstance () throws RoomRepositoryException {
		if (instance==null) {
			instance = new RoomRepository ();
		}
		return instance;
	}


	private RoomRepository () throws RoomRepositoryException {
		String get = "SELECT * FROM rooms WHERE ROOMID=?";
		String put = "INSERT INTO rooms VALUES ( ?,?,?,?,?,? )";
		String rem = "DELETE FROM rooms WHERE ROOMID=?";
		String upd = "UPDATE rooms SET COMMUNITYID=?, NAME=?, ADDR=?, TYPE=?, STATUS=? WHERE ROOMID=?";
		String all = "SELECT * FROM rooms ORDER BY ROOMID";
		// get the file name of the database
		XGSPConfiguration xgsp_conf = XGSPConfiguration.getInstance ();

		try {
			DBConnection db = new DBConnection ();
			connection = db.getConnection ();

			getStmt = connection.prepareStatement (get);
			putStmt = connection.prepareStatement (put);
			remStmt = connection.prepareStatement (rem);
			updStmt = connection.prepareStatement (upd);
			getAllStmt = connection.prepareStatement (all);
		} catch (ClassNotFoundException e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			throw new RoomRepositoryException ("No Driver Available!");
		} catch (SQLException e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			throw new RoomRepositoryException (e.getMessage ());
		} catch (Exception e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			e.printStackTrace ();
		}
	}


	private RoomBean makeRoom (ResultSet results) throws RoomRepositoryException {
		try {
			RoomBean Room = new RoomBean ();
			Room.setRoomid (results.getString ("ROOMID"));
			Room.setCommunityid (results.getString ("COMMUNITYID"));
			Room.setRoomname (results.getString ("NAME"));
			Room.setRoomaddr (results.getString ("ADDR"));
			Room.setRoomtype (results.getString ("TYPE"));
			Room.setRoomstatus (results.getString ("STATUS"));
			return Room;
		} catch (SQLException e) {
			throw new RoomRepositoryException (e.getMessage ());
		}
	}


	public RoomBean getRoom (String Roomid) throws UnknownRoomException,
			 RoomRepositoryException {
		try {
			ResultSet results;
			synchronized (getStmt) {
				getStmt.clearParameters ();
				getStmt.setString (1, Roomid);
				results = getStmt.executeQuery ();
			}
			if (results.next ()) {
				return makeRoom (results);
			} else {
				throw new UnknownRoomException ("could not find Room"+Roomid);
			}
		} catch (SQLException e) {
			throw new RoomRepositoryException (e.getMessage ());
		}
	}


	// Get all Coomunity records
	public RoomBean[] getRooms () throws RoomRepositoryException {
		try {
			ResultSet results;
			Collection rooms = new ArrayList ();
			synchronized (getAllStmt) {
				results = getAllStmt.executeQuery ();
			}
			RoomBean room;
			while (results.next ()) {
				rooms.add (makeRoom (results));
			}
			return (RoomBean[]) rooms.toArray (new RoomBean[0]);
		} catch (SQLException e) {
			throw new RoomRepositoryException (e.getMessage ());
		}
	}


	public void put (RoomBean Room) throws
			 RoomRepositoryException {
		try {
			synchronized (putStmt) {
				putStmt.clearParameters ();
				putStmt.setString (1, Room.getRoomid ());
				putStmt.setString (2, Room.getCommunityid ());
				putStmt.setString (3, Room.getRoomname ());
				putStmt.setString (4, Room.getRoomaddr ());
				putStmt.setString (5, Room.getRoomtype ());
				putStmt.setString (6, Room.getRoomstatus ());
				putStmt.executeUpdate ();
			}
		} catch (SQLException e) {
			throw new RoomRepositoryException (e.getMessage ());
		}
	}


	public void removeRoom (String roomid) throws
			 RoomRepositoryException {
		try {
			synchronized (remStmt) {
				remStmt.clearParameters ();
				remStmt.setString (1, roomid);
				int rowsChanged = remStmt.executeUpdate ();
				if (rowsChanged<1) {
					throw new UnknownRoomException ("Can't delete Room# "+roomid);
				}
			}
		} catch (SQLException e) {
			throw new RoomRepositoryException (e.getMessage ());
		}
	}


	// Update the old community record
	public void update (RoomBean room) throws
			 UnknownRoomException, RoomRepositoryException {
		try {
			synchronized (updStmt) {
				updStmt.clearParameters ();
				updStmt.setString (6, room.getRoomid ());
				updStmt.setString (1, room.getCommunityid ());
				updStmt.setString (2, room.getRoomname ());
				updStmt.setString (3, room.getRoomaddr ());
				updStmt.setString (4, room.getRoomtype ());
				updStmt.setString (5, room.getRoomstatus ());
				int rowsChanged = updStmt.executeUpdate ();
				if (rowsChanged<1) {
					throw new UnknownRoomException ("Could not find Room# "+
							 room.getRoomid ());
				}
			}
		} catch (SQLException e) {
			throw new RoomRepositoryException (e.getMessage ());
		}
	}

}
