/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

import java.sql.*;
import java.util.*;
import java.util.logging.*;


public class SessionParameters {

	private String daystring; // for a certain day selection
	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	//
	private String dayPassString_from;
	private String dayPassString_to;
	private String dayPassString_a_day; // for a certain day selection
	private String time; // past, future, ongoing, etc ...
	private String room;
	private String whosemeetings;
//     private String userid;
	private String sessionid;
	private String sessionname;
	private String sessionowner;
	private String sessionchair;
	private String sessiondes;
	private String sessionplace;

	private Calendar currentCal;
	private String currentCalString;

	public SessionParameters () {
		daystring = null; // for a certain day selection

		//
		dayPassString_from = null;
		dayPassString_to = null;
		time = null; // past, future, ongoing, etc ...

		//
		room = null;
		//
		sessionid = null;
		sessionname = null;
		sessionowner = null;
		sessionchair = null;
		sessiondes = null;
		sessionplace = null;

		currentCal = new GregorianCalendar ();
		currentCal.setTime (new java.util.Date (System.currentTimeMillis ()));
		currentCalString = (new java.util.Date (System.currentTimeMillis ())).
					    toString ();
	}


	public void initialize () {
		daystring = null; // for a certain day selection

		//
		dayPassString_from = null;
		dayPassString_to = null;
		time = null; // past, future, ongoing, etc ...

		//
		room = null;
		//
		sessionid = null;
		sessionname = null;
		sessionowner = null;
		sessionchair = null;
		sessiondes = null;
		sessionplace = null;

		currentCal = new GregorianCalendar ();
		currentCal.setTime (new java.util.Date (System.currentTimeMillis ()));
	}


	private boolean isCriteriaAfterThanSessionBean (String criteria,
			 SessionBean sb) {

		// first thing to compare ...
		Calendar calendar2 = new GregorianCalendar ();

		calendar2.setTime (new java.util.Date (criteria));
		//calendar2.setTime (new java.util.Date (Long.parseLong (criteria)));

		// second thing to compare ...
		Timestamp ts = sb.getSessionstart ();
		Calendar calTS = new GregorianCalendar ();
		calTS.setTimeInMillis (ts.getTime ());

		if (calendar2.after ((Calendar) calTS)) {
			// true if the current time of calendar2 is after ...
			return true;
		}

		return false;
	}


	private boolean isCriteriaAfterThanSessionBean (Calendar calendar2,
			 SessionBean sb) {

		// second thing to compare ...
		Timestamp ts = sb.getSessionstart ();
		Calendar calTS = new GregorianCalendar ();
		calTS.setTimeInMillis (ts.getTime ());

		if (calendar2.after ((Calendar) calTS)) {
			// true if the current time of calendar2 is after ...
			return true;
		}

		return false;
	}


	private boolean isCriteriaAfterThanSessionBean (String criteria, Timestamp ts) {

		// first thing to compare ...
		Calendar calendar2 = new GregorianCalendar ();

		//calendar2.setTime (new java.util.Date (Long.parseLong (criteria)));
		calendar2.setTime (new java.util.Date (criteria));

		// second thing to compare ...
		Calendar calTS = new GregorianCalendar ();
		calTS.setTimeInMillis (ts.getTime ());

		if (calendar2.after ((Calendar) calTS)) {
			// true if the current time of calendar2 is after ...
			return true;
		}

		return false;
	}


	private boolean isCriteriaAfterThanSessionBean (Calendar calendar2,
			 Timestamp ts) {

		// second thing to compare ...
		Calendar calTS = new GregorianCalendar ();
		calTS.setTimeInMillis (ts.getTime ());

		if (calendar2.after ((Calendar) calTS)) {
			// true if the current time of calendar2 is after ...
			return true;
		}

		return false;
	}


	private boolean isSameDay (String criteria, SessionBean sb) {
		// first thing to compare ...
		Calendar calendar2 = new GregorianCalendar ();

		//calendar2.setTime (new java.util.Date (Long.parseLong (criteria)));
		//calendar2.setTime (new java.util.Date (criteria));

		// second thing to compare ...
		Timestamp ts = sb.getSessionstart ();
		Calendar calTS = new GregorianCalendar ();
		calTS.setTimeInMillis (ts.getTime ());

		if (Utility.isSameDay (calendar2, calTS)) {
			return true;
		}
		return false;
	}


	public boolean validateToSearchParameters (SessionBean sb) {

		//------------------------------------------------------------------
		if (sb==null) {
			System.out.println ("sb = null");
			return false; //actually, this can be exception ...
		}

		//------------------------------------------------------------------
		if (daystring!=null) {

			java.util.Date date = new java.util.Date (Long.parseLong (daystring));
			if (this.isSameDay (date.toString (), sb)) {
				System.out.println ("pass: daystring");
				return true;

			}
			System.out.println ("fail: daystring");
			return false;
		}

		//-------------------------------------------------------------------
		if (dayPassString_to!=null) {
			// dayPassString_to --> should be after --> sb.getSessionstart ()
			// calendar2 --> should be after -->  calTS
			if (!this.isCriteriaAfterThanSessionBean (dayPassString_to, sb)) {
				return false;
			}
		}

		//------------------------------------------------------------------
		if (dayPassString_from!=null) {
			// dayPassString_from --> should be before --> sb.getSessionstart ()
			// calendar2 --> should be before -->  calTS
			if (this.isCriteriaAfterThanSessionBean (dayPassString_from, sb)) {
				return false;
			}
		}

		//------------------------------------------------------------------
		if (time!=null) {

			if (!time.equalsIgnoreCase ("all")) {

				if (time.equalsIgnoreCase ("past")) {
					if (!this.isCriteriaAfterThanSessionBean (currentCal, sb)) {
						return false;
					}
				}

				if (time.equalsIgnoreCase ("future")) {
					if (this.isCriteriaAfterThanSessionBean (currentCal, sb)) {
						return false;
					}
				}

				if (time.equalsIgnoreCase ("ongoing")) {
					// !(cuttenttime is before starting time of ts) --> false
					if (!this.isCriteriaAfterThanSessionBean (currentCal,
							 sb.getSessionstart ())) {
						return false;
					}

					// !(cuttenttime is after ending time of ts) --> false
					if (this.isCriteriaAfterThanSessionBean (currentCal, sb.getSessionend ())) {
						return false;
					}
				}

				if (time.equalsIgnoreCase ("today")) {
					if (!this.isSameDay (currentCalString, sb)) {

						return false;
					}
				}
			}
		}

		//------------------------------------------------------------------
		if (room!=null) {
			if (!room.equalsIgnoreCase ("all")) {

				boolean found = false;
				Vector v = sb.getSessionrooms ();
				Iterator itr = v.iterator ();
				String s = "";

				while (itr.hasNext ()) {
					s = (String) itr.next ();
					if (s.equalsIgnoreCase (room)) {
						found = true;
						break;
					}
				}

				if (!found) {
					return false;
				}
			}
		}

		//------------------------------------------------------------------
		if (sessionid!=null) {
			if (!sessionid.equalsIgnoreCase (sb.getSessionid ())) {
				return false;
			}
		}
		//------------------------------------------------------------------
		if (sessionname!=null) {
			if (!sessionname.equalsIgnoreCase (sb.getSessionname ())) {
				return false;
			}
		}
		//------------------------------------------------------------------
		if (sessionowner!=null) {
			if (!sessionowner.equalsIgnoreCase ("all")) {
				// we need a userid now !!!!!!!!!!!!!!!!

				if (!this.sessionowner.equalsIgnoreCase (sb.getSessionowner ())) {
					return false;
				}
			}
		}
		//------------------------------------------------------------------
		if (sessionchair!=null) {
			if (!sessionchair.equalsIgnoreCase (sb.getSessionchair ())) {
				return false;
			}
		}
		//------------------------------------------------------------------
		if (sessiondes!=null) {
			if (!sessiondes.equalsIgnoreCase (sb.getSessiondes ())) {
				return false;
			}
		}
		//------------------------------------------------------------------
		if (sessionplace!=null) {
			if (!sessionplace.equalsIgnoreCase (sb.getSessionplace ())) {
				return false;
			}
		}

		//------------------------------------------------------------------
		return true;
		//------------------------------------------------------------------
	}


	public void print () {
		logger.fine ("daystring           : "+daystring);
		logger.fine ("room                : "+room);
		//
		logger.fine ("dayPassString_from  : "+dayPassString_from+
				   " dayPassString_to    : "+dayPassString_to);
		logger.fine ("dayPassString_a_day : "+dayPassString_a_day); // for a certain day selection
		logger.fine ("time                : "+time); // past, future, ongoing, etc ...
		//
		logger.fine ("sessionid  : "+sessionid+" sessionname  : "+sessionname);
		logger.fine ("sessionowner : "+sessionowner+" sessionchair  : "+
				   sessionchair);
		logger.fine ("sessionchair : "+sessionchair+"sessiondes : "+sessiondes+
				   "sessionplace : "+sessionplace);
	}


	public static void main (String[] args) {
		SessionParameters s = new SessionParameters ();
	}


	public String getDayPassString_from () {
		return dayPassString_from;
	}


	public String getDayPassString_to () {
		return dayPassString_to;
	}


	public String getDaystring () {
		return daystring;
	}


	public String getSessionchair () {
		return sessionchair;
	}


	public String getSessiondes () {
		return sessiondes;
	}


	public String getSessionid () {
		return sessionid;
	}


	public String getSessionname () {
		return sessionname;
	}


	public String getSessionowner () {
		return sessionowner;
	}


	public String getSessionplace () {
		return sessionplace;
	}


	public String getTime () {
		return time;
	}


	public void setTime (String time) {
		this.time = time;
	}


	public void setSessionplace (String sessionplace) {
		this.sessionplace = sessionplace;
	}


	public void setSessionowner (String sessionowner) {
		this.sessionowner = sessionowner;
	}


	public void setSessionname (String sessionname) {
		this.sessionname = sessionname;
	}


	public void setSessionid (String sessionid) {
		this.sessionid = sessionid;
	}


	public void setSessiondes (String sessiondes) {
		this.sessiondes = sessiondes;
	}


	public void setSessionchair (String sessionchair) {
		this.sessionchair = sessionchair;
	}


	public void setDaystring (String daystring) {
		this.daystring = daystring;
	}


	public void setDayPassString_to (String dayPassString_to) {
		this.dayPassString_to = dayPassString_to;
	}


	public void setDayPassString_from (String dayPassString_from) {
		this.dayPassString_from = dayPassString_from;
	}


	public String getRoom () {
		return room;
	}


	public void setRoom (String room) {
		this.room = room;
	}


	public String getWhosemeetings () {
		return whosemeetings;
	}


	public void setWhosemeetings (String whosemeetings) {
		this.whosemeetings = whosemeetings;
	}

}