/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

import java.sql.*;
import java.io.*;
import java.util.*;


// Storage Module
/** Methods
 ** SessionRepository()
 ** makeSession(html-result), getSession(session-id), put(session-bean)
 ** sessionid, sessionname,sessionowner,sessionchair,sessiondes,sessionstart,sessionend,sessionplace
 **/
public class SessionRepository {
	private static SessionRepository instance;
	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	private Connection connection;
	private PreparedStatement getStmt;
	private PreparedStatement putStmt;
	private PreparedStatement remStmt;
	private PreparedStatement updStmt;
	private PreparedStatement getAllStmt;
	private PreparedStatement findStmt;
	private PreparedStatement getPersistStmt;

	public static SessionRepository getInstance () throws
			 SessionRepositoryException {
		if (instance==null) {
			instance = new SessionRepository ();
		}
		return instance;
	}


	private SessionRepository () throws SessionRepositoryException {
		String get = "SELECT * FROM sessions WHERE SESSIONID=?";
		String put = "INSERT INTO sessions VALUES ( ?,?,?,?,?,?,?,? )";
		String rem = "DELETE FROM sessions WHERE SESSIONID=?";
		String upd =
				 "UPDATE sessions SET NAME=?, OWNER=?, CHAIR=?, DES=?, STARTTIME=?, ENDTIME=?, PLACE=? WHERE SESSIONID=?";
		String all = "SELECT * FROM sessions ORDER BY SESSIONID";
		String find =
				 "SELECT * FROM sessions WHERE STARTTIME < ? AND STARTTIME >= ? ";
		String findEnd = "SELECT * FROM sessions WHERE ENDTIME >= ? ";
		String persist = "SELECT * FROM sessions WHERE DES=?";

		try {
			DBConnection db = new DBConnection ();
			connection = db.getConnection ();

			getStmt = connection.prepareStatement (get);
			putStmt = connection.prepareStatement (put);
			remStmt = connection.prepareStatement (rem);
			updStmt = connection.prepareStatement (upd);
			getAllStmt = connection.prepareStatement (all);
			findStmt = connection.prepareStatement (find);
			getPersistStmt = connection.prepareStatement (persist);
		} catch (ClassNotFoundException e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			throw new SessionRepositoryException ("No Driver Available!");
		} catch (SQLException e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			throw new SessionRepositoryException (e.getMessage ());
		} catch (Exception e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			e.printStackTrace ();
		}
	}


	private SessionBean makeSession (ResultSet results) throws
			 SessionRepositoryException {
		try {
			SessionBean Session = new SessionBean ();
			Session.setSessionid (results.getString ("SESSIONID"));
			Session.setSessionname (results.getString ("NAME"));
			Session.setSessionowner (results.getString ("OWNER"));
			Session.setSessionchair (results.getString ("CHAIR"));
			Session.setSessiondes (results.getString ("DES"));
			Session.setSessionstart (results.getTimestamp ("STARTTIME"));
			Session.setSessionend (results.getTimestamp ("ENDTIME"));
			Session.setSessionplace (results.getString ("PLACE"));
			return Session;
		} catch (SQLException e) {
			throw new SessionRepositoryException (e.getMessage ());
		}
	}


	public SessionBean getSession (String Sessionid) throws
			 UnknownSessionException, SessionRepositoryException {
		try {
			ResultSet results;
			synchronized (getStmt) {
				getStmt.clearParameters ();
				getStmt.setString (1, Sessionid);
				results = getStmt.executeQuery ();
			}
			if (results.next ()) {
				return makeSession (results);
			} else {
				throw new UnknownSessionException ("could not find Session"+Sessionid);
			}
		} catch (SQLException e) {
			throw new SessionRepositoryException (e.getMessage ());
		}
	}


	// Get all Coomunity records
	public SessionBean[] getSessions () throws SessionRepositoryException {
		try {
			ResultSet results;
			Collection sessions = new ArrayList ();
			synchronized (getAllStmt) {
				results = getAllStmt.executeQuery ();
			}
			SessionBean session;
			while (results.next ()) {
				sessions.add (makeSession (results));
			}
			return (SessionBean[]) sessions.toArray (new SessionBean[0]);
		} catch (SQLException e) {
			throw new SessionRepositoryException (e.getMessage ());
		}
	}


	/**
	 * Harun's Method
	 * Find the sessions in a specific day
	 * @param daystring : string represtation of a day
	 */
	public SessionBean[] getSessionsInASpecificDay (String daystring) throws
			 SessionRepositoryException {
		Calendar calendar2 = new GregorianCalendar (); //	calendarDateThatWasSelected
		calendar2.setTime (new java.util.Date (Long.parseLong (daystring))); // first thing to compare ...

		SessionBean[] sbs = this.getSessions ();
		Collection sessions = new ArrayList ();

		for (int i = 0; i<sbs.length; i++) {
			SessionBean sb = sbs[i];

			Timestamp ts = sb.getSessionstart ();
			Calendar calTS = new GregorianCalendar (); //	calendarDateOfThisSpesificRecord
			calTS.setTimeInMillis (ts.getTime ()); // second thing to compare ...

			if (Utility.isSameDay (calendar2, calTS)) {
				sessions.add (sb);
			}
		}
		return (SessionBean[]) sessions.toArray (new SessionBean[0]);
	}


	/**
	 * Harun's method
	 * combo query of the sessions
	 */
	public SessionBean[] getSessions (SessionParameters sp) throws
			 SessionRepositoryException {
		sp.print ();
		SessionBean[] sbs = this.getSessions ();
		Collection sessions = new ArrayList ();
		SessionBean sb;

		for (int i = 0; i<sbs.length; i++) {
			sb = sbs[i];
			if (sp.validateToSearchParameters (sb)) {
				sessions.add (sb);
			}
		}
		sp.initialize ();
		return (SessionBean[]) sessions.toArray (new SessionBean[0]);
	}


	/**
	 * Find the sessions in the day
	 * <p> this method searches the session during the period < date-before, date-after >
	 * @param date-after : after timestamp
	 * @param date-before: before timestamp
	 */
	public SessionBean[] findSessions (Timestamp dateafter, Timestamp databefore) throws
			 SessionRepositoryException {
		try {
			ResultSet results;
			Collection sessions = new ArrayList ();

			synchronized (findStmt) {
				findStmt.clearParameters ();
				findStmt.setTimestamp (1, dateafter);
				findStmt.setTimestamp (2, databefore);
				results = findStmt.executeQuery ();
			}
			SessionBean session;
			while (results.next ()) {
				sessions.add (makeSession (results));
			}
			return (SessionBean[]) sessions.toArray (new SessionBean[0]);
		} catch (SQLException e) {
			throw new SessionRepositoryException (e.getMessage ());
		}
	}


	// Get the permanant Sessions
	public SessionBean[] getPersistSessions () throws SessionRepositoryException {
		try {
			ResultSet results;
			Collection sessions = new ArrayList ();
			synchronized (getPersistStmt) {
				getPersistStmt.clearParameters ();
				getPersistStmt.setString (1, "PERMANENT");
				results = getPersistStmt.executeQuery ();
			}
			SessionBean session;
			while (results.next ()) {
				sessions.add (makeSession (results));
			}
			return (SessionBean[]) sessions.toArray (new SessionBean[0]);
		} catch (SQLException e) {
			throw new SessionRepositoryException (e.getMessage ());
		}
	}


	// Put a new sessionbean into database
	public void put (SessionBean Session) throws
			 SessionRepositoryException {
		try {
			synchronized (putStmt) {
				putStmt.clearParameters ();
				putStmt.setString (1, Session.getSessionid ());
				putStmt.setString (2, Session.getSessionname ());
				putStmt.setString (3, Session.getSessionowner ());
				putStmt.setString (4, Session.getSessionchair ());
				putStmt.setString (5, Session.getSessiondes ());
				putStmt.setTimestamp (6, Session.getSessionstart ());
				putStmt.setTimestamp (7, Session.getSessionend ());
				putStmt.setString (8, Session.getSessionplace ());

				putStmt.executeUpdate ();
			}
		} catch (SQLException e) {
			throw new SessionRepositoryException (e.getMessage ());
		}
	}


	// Delete Session Record
	public void removeSession (String sessid) throws
			 SessionRepositoryException {
		try {
			synchronized (remStmt) {
				remStmt.clearParameters ();
				remStmt.setString (1, sessid);
				int rowsChanged = remStmt.executeUpdate ();
				if (rowsChanged<1) {
					throw new UnknownSessionException ("Can't delete Session# "+sessid);
				}
			}
		} catch (SQLException e) {
			throw new SessionRepositoryException (e.getMessage ());
		}
	}


	// Update the old community record
	public void update (SessionBean sess) throws
			 UnknownSessionException, SessionRepositoryException {
		try {
			synchronized (updStmt) {
				updStmt.clearParameters ();
				updStmt.setString (8, sess.getSessionid ());
				updStmt.setString (1, sess.getSessionname ());
				updStmt.setString (2, sess.getSessionowner ());
				updStmt.setString (3, sess.getSessionchair ());
				updStmt.setString (4, sess.getSessiondes ());
				updStmt.setTimestamp (5, sess.getSessionstart ());
				updStmt.setTimestamp (6, sess.getSessionend ());
				updStmt.setString (7, sess.getSessionplace ());
				int rowsChanged = updStmt.executeUpdate ();
				if (rowsChanged<1) {
					throw new UnknownSessionException ("Could not find Session# "+
							 sess.getSessionid ());
				}
			}
		} catch (SQLException e) {
			throw new SessionRepositoryException (e.getMessage ());
		}
	}
}
