/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

import cgl.globalmmcs.webserver.admin.meetingMgt.getSession;
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import java.util.logging.*;
import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import cgl.globalmmcs.webserver.sessionmgt.*;
import java.util.logging.*;

import cgl.globalmmcs.webserver.user.*;

import cgl.globalmmcs.webserver.admin.meetingMgt.GetAllSession;
import cgl.globalmmcs.webserver.admin.meetingMgt.list_today_session;
import cgl.globalmmcs.webserver.admin.meetingMgt.AddSession;


// Login Command

public class SessionServlet extends HttpServlet {

	private HashMap commands;
	private String Error = "error.jsp";
	private String NoLogin = "Nologin.jsp";
	private String jspdir = "/";

	private ArrayList logInComms; // keeps the portal commands which needs logIn
	boolean ok;
	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	//Initialize global variables
	public void init (ServletConfig config) throws ServletException {
		InputStream is = null;
		try {
			is = new FileInputStream ("webserver.log.properties");
		} catch (FileNotFoundException ex) {
			ex.printStackTrace ();
		}

		LogManager lm = LogManager.getLogManager ();
		try {
			lm.readConfiguration (is);
		} catch (Exception ex) {
			ex.printStackTrace ();
		}

//-------------

		super.init (config);
		initCommands ();
		logInCommands ();
//-------------

		String H323GW = getInitParameter ("H323GW_ADDR");
		String SIPGW = getInitParameter ("SIPGW_ADDR");
		String Sv_Addr = getInitParameter ("SESSIONSV_ADDR");
		String DB_Path = getInitParameter ("DATABASE_PATH");
		String NB_Addr = getInitParameter ("NARADAB_ADDR");
		String NB_Port = getInitParameter ("NARADAB_PORT");

		logger.info ("H323GW :"+H323GW);
		logger.info ("SIPGW :"+SIPGW);
		logger.info ("Sv_Addr :"+Sv_Addr);
		logger.info ("DB_Path :"+DB_Path);
		logger.info ("NB_Addr :"+NB_Addr);
		logger.info ("NB_Port :"+NB_Port);

		//
		XGSPConfiguration xgsp_conf = XGSPConfiguration.getInstance ();
		xgsp_conf.setH323GW (H323GW);
		xgsp_conf.setSIPGW (SIPGW);
		xgsp_conf.setSessionSv (Sv_Addr);
//harun          xgsp_conf.setDatabaseAddr (DB_Path);
		xgsp_conf.setNBAddr (NB_Addr);
		xgsp_conf.setNBPort (NB_Port);

		// start the hearbeat monitor
		HeartBeatProcess heartbeat_mon = new HeartBeatProcess ();
		heartbeat_mon.start ();

		// start ActiveSessionDemon here:
		XGSPAVSessionManager manager = XGSPAVSessionManager.getInstance ();
		manager.StartSessionDemo ();
		logger.info ("no exception in SessionServlet");

	}


	// Main Control Routine
	public void service (HttpServletRequest req, HttpServletResponse res) throws
			 ServletException, IOException {

		logger.info ("SessionServlet ... service ... enter");

		req.setAttribute ("servletName", "portal");

		//
		String cmd_str = null;
		UserBean user;
		HttpSession session;

		//
		session = req.getSession (true);
		user = (UserBean) session.getAttribute ("userLogged");

		//
		cmd_str = req.getParameter ("cmd");

		//
		if (user==null) {
			logger.info ("user null");
			if (cmd_str!=null) {
				if (!((cmd_str.equalsIgnoreCase ("do-login"))||
					 (cmd_str.equalsIgnoreCase ("do-register"))||
					 (cmd_str.equalsIgnoreCase ("register"))
					 )) {
					cmd_str = "login";
				}
			}
		}

		if (user!=null) {
			logger.info ("user : "+user);
		}

		//
		String next = null;
		try {
			Command cmd = lookupCommand (cmd_str);

			if (cmd==null) {
				logger.severe ("cmd == null");
				cmd = new NullCommand ("error.jsp");
			}

			if (req==null) {
				logger.severe ("req == null");
			}

			next = cmd.execute (req);

			if (next==null) {
				logger.severe ("next == null");
			}

			logger.info ("cmd_str : "+cmd_str);
			logger.info ("next    : "+next);

		} catch (Exception e) {
			logger.severe (e.getMessage ());
			logger.severe (e.toString ());
			logger.severe ("cmd_str : "+cmd_str);

			req.setAttribute ("javax.servlet.jsp.jspException", e);
			next = Error;
			logger.severe ("next    : "+next);
		}

		RequestDispatcher rd;

		// harun ...
		String login_status_changed = (String) session.getAttribute (
				 "login_status_changed");

		if (false) {
			if (login_status_changed!=null) {
				if (login_status_changed.equalsIgnoreCase ("yes")) {
					//
					// refresh nav_portal here ...
					//
					//
					rd = getServletContext ().getRequestDispatcher (jspdir+"portal.jsp");
					rd.forward (req, res);

					session.setAttribute ("login_status_changed", "no");
				}
			}
		}

		logger.info ("jspdir+next :"+jspdir+next);
		logger.info ("next :"+next);

		rd = getServletContext ().getRequestDispatcher (jspdir+next);
		rd.forward (req, res);
		logger.info ("SessionServlet ... service ... exit");

		/*   String cmdName = "unknown";
		   String next;
		   try {
		 cmdName = req.getParameter("cmd");
		 Command cmd = lookupCommand(req.getParameter("cmd"));
		 // add-session, join-session need the login
		 ok = checkLogIn(req, cmdName);
		 if (ok) {
		   next = cmd.execute(req);
		 }
		 else {
		   next = NoLogin;
		 }
		   }
		   catch (CommandException e) {
		 req.setAttribute("javax.servlet.jsp.jspException", e);
		 next = Error;
		 logger.info("command execution error!!" + cmdName);
		   }
		   catch (UnknownCommandException e) {
		 req.setAttribute("javax.servlet.jsp.jspException", e);
		 next = Error;
		 logger.info("can't find the command " + cmdName);
		   }
		   RequestDispatcher rd;
		   rd = getServletContext().getRequestDispatcher(jspdir + next);
		   rd.forward(req, res);*/
	}


	/**
	 * check the logIn status before the user can start to use the system
	 * check the following cmd: edit-user-yourself,
	 * @return the next JSP page
	 */
	private boolean checkLogIn (HttpServletRequest req, String cmdName) {
		boolean ok = true;
		// add-session, join-session need the login
		if (logInComms.contains (cmdName)) {
			HttpSession session = req.getSession (false);
			try {
				if (session==null||session.getValue ("user")==null) {
					ok = false;
				}
			} catch (IllegalStateException e) {
				ok = false;
			}
		}
		return (ok);
	}


	private Command lookupCommand (String cmd) throws CommandException {
		logger.info ("SessionServlet ... lookupCommand ... enter");
		Command c;

		try {
			if (commands==null) {
				logger.severe ("commands is null !!!!!!");
			}
			logger.info ("cmd : "+cmd);

			if (cmd==null) {
				logger.info ("cmd is null");
				logger.info ("cmd is NOW ACCEPTED TO BE: admin-main");

				c = (Command) commands.get ("portal-main");

				//logger.info ("c.getClass().getName() : "+c.getClass ().getName ()); //harun ---
				logger.info ("SessionServlet ... lookupCommand ... exit");
				return c;
			}

			if (commands.containsKey (cmd.toLowerCase ())) {
				logger.info ("cmd is NOT null");
				logger.info ("cmd is : "+cmd);

				c = (Command) commands.get (cmd.toLowerCase ());

				logger.info ("c.getClass().getName() : "+c.getClass ().getName ());
				logger.info ("SessionServlet ... lookupCommand ... exit");
				return c;
			}

			logger.info ("throw new CommandException (\"Invalid Command Identifier\")");
			logger.info (
					 "SessionServlet ... lookupCommand ... exit with CommandException");
		} catch (Exception e) {
			logger.info ("harun");
			e.printStackTrace ();

		}

		throw new CommandException ("Invalid Command Identifier");

		/*    if (cmd == null) {
		  //cmd = "main-menu";
		  //cmd = "manage-community";
		  cmd = "portal-main";
		    }
		    if (commands.containsKey(cmd.toLowerCase())) {
		  return (Command) commands.get(cmd.toLowerCase());
		    }
		    else {
		  throw new UnknownCommandException("Invalid Command Identifier");
		    }*/
	}


	// InitCommands: build command Hasp table
	private void initCommands () {
		commands = new HashMap ();

		//commands.put("portal-main", new GetAllSession("portal.jsp"));
		commands.put ("portal-main", new NullCommand ("portal.jsp"));

		/** login, register command **/
		//commands.put("main-menu", new NullCommand("login.jsp"));
		commands.put ("main-menu", new NullCommand ("menu.jsp"));
		commands.put ("login", new NullCommand ("login.jsp"));
		commands.put ("register", new NullCommand ("register.jsp"));
		commands.put ("do-login", new LogIn ("welcome.jsp"));
		commands.put ("logout", new LogOut ("login.jsp")); // end.jsp??
		commands.put ("do-register", new RegisterComm ("login.jsp"));

		// dedicate command for building portlet
		commands.put ("dosession", new LogInSessionComm ("join_session.jsp"));

		// user profile updation
		commands.put ("edit-user-yourself",
				    new getUserYourself ("edituser_yourself.jsp")); //Harun added
		commands.put ("update-user-yourself", new editUser ("home_portal.jsp")); //Harun added

		// Make Meeting Schedule ( Not completed yet !! )
		commands.put ("add-session", new AddSession ("add_session_result.jsp"));
		// list all the meeting schedules
		commands.put ("list-of-meetings",
				    new GetAllSession ("ListAllSessionsUSER.jsp"));
		// list today's meeting for joining
		commands.put ("list-session", new list_today_session ("join_session.jsp"));
		// join meeting
		commands.put ("join-session", new joinSession ("AVenterRoom.jsp"));

		// deprecated commands for old version of web server
		//commands.put("join-session", new joinSession("enterRoom.jsp"));
		//commands.put("enter-room", new enterRoom("SessionWork.jsp"));

//---
		// Make Meeting Schedule
		commands.put ("add-session-user", new AddSession ("ListAllSessionsUSER.jsp"));
		commands.put ("manage-session", new GetAllSession ("managesessionUSER.jsp"));
		commands.put ("get-session", new getSession ("sessioninfoUSER.jsp"));
		commands.put ("add-session-empty", new NullCommand ("addsessionUSER.jsp"));

		// Join Meeting
		commands.put ("list-room", new NullCommand ("ListRoomUSER.jsp"));

	}


	// set the portal commands which need login at first
	private void logInCommands () {
		logInComms = new ArrayList ();
		logInComms.add ("edit-user-yourself");
		logInComms.add ("list-session");
		logInComms.add ("list-of-meetings");
	}
}