/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

import org.jdom.*;


public class StreamBean {
	/** stream bean field
	 ** 1) SSRC:
	 ** 2) stream-type: video/audio
	 ** 3) UserName
	 ** 4) Stream description
	 ** 5) ImgPath --> snapshot img
	 ** 6) EventType --> NewStreamEvent, ByeEvent, ActiveToPassive, PassiveToActive
	 **/
	private String SSRC;
	private int StreamType; // 0: video; 1: audio
	private String UserName;
	private String StrDescription;
	private String imgpath;
	private boolean mixed;

	private String eventType;

	private String sessionID;
	private boolean active; // true: active; false: in-active
	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	public StreamBean () {
		this.SSRC = "";
		StreamType = 0;
		UserName = "";
		StrDescription = "";
		imgpath = "";
		mixed = false;
		eventType = null;
		active = true;
	}


	public void setSsrc (String ssrc) {
		this.SSRC = ssrc;
	}


	public String getSsrc () {
		return this.SSRC;
	}


	public void setStreamtype (int str_type) {
		this.StreamType = str_type;
	}


	public String getStreamtype () {
		if (this.StreamType==0) {
			return ("video");
		} else {
			return ("audio");
		}
	}


	public void setUsername (String username) {
		this.UserName = username;
	}


	public String getUsername () {
		return this.UserName;
	}


	public void setStrdescription (String str_des) {
		this.StrDescription = str_des;
	}


	public String getStrdescription () {
		return this.StrDescription;
	}


	public void setImgpath (String imgpath) {
		//this.imgpath = imgpath+"/"+this.SSRC+".JPEG";
		this.imgpath = imgpath;
	}


	public String getImgpath () {
		return this.imgpath;
	}


	public String getEventType () {
		return this.eventType;
	}


	public void setMixed (boolean value) {
		mixed = value;
	}


	public boolean getMixed () {
		return mixed;
	}


	public void setActive (boolean activeFlag) {
		this.active = activeFlag;
	}


	public boolean getActive () {
		return this.active;
	}


	/**
	 * this method is out of date
	 */
	public void setStreamBean (Element clientEl, String mediaType) {
		this.setUsername (clientEl.getChild ("username").getTextTrim ());
		this.setSsrc ((clientEl.getChild ("ssrc").getTextTrim ()));
		this.setStrdescription (clientEl.getChild ("description").getTextTrim ());
		if (mediaType.equals ("video")) {
			this.setStreamtype (0);
		} else {
			this.setStreamtype (1);
		}
	}


	/**
	 * this method parses the StreamEvent XML message and create StreamEvent object
	 */
	public void setStreamBean (Element clientEl) {
		this.setSessionID (clientEl.getChild ("SessionID").getTextTrim ());
		eventType = (clientEl.getChild ("EventType").getTextTrim ());
		String stmType = (clientEl.getChild ("MediaType").getTextTrim ());
		// go to <Stream> .. </Stream>: <UserName>, <SSRC>, <Description>
		Element Strm = clientEl.getChild ("Stream");
		this.setUsername (Strm.getChild ("UserName").getTextTrim ());
		this.setSsrc ((Strm.getChild ("SSRC").getTextTrim ()));
		this.setStrdescription (Strm.getChild ("Description").getTextTrim ());

		this.setImgpath (clientEl.getChild ("ImagePath").getTextTrim ());
		if (stmType.equalsIgnoreCase ("video")) {
			this.setStreamtype (0);
		} else {
			this.setStreamtype (1);
		}
		// since in SessionMgt , UserName is also used. Need to check if whether it is null
		// if it is null, set it to unknown
		if (this.UserName==null) {
			this.UserName = "UnKnown";
		}
	}


	public void setSessionID (String sessionID) {
		this.sessionID = sessionID;
	}


	public String getSessionID () {
		return this.sessionID;
	}

}