/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

import java.sql.*;
import java.io.*;
import java.util.*;


// Storage Module
/** UserBean field
 userid, passwd, username, user type, contact info ( email, telephone, .. )
 **/
public class UserRepository {
	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	private static UserRepository instance;
	/*
	  private static final String Driver =
	  "com.borland.datastore.jdbc.DataStoreDriver";
	  private static final String user = "guest";
	  private static final String pass = "guest";
	  private static String dbURL = "jdbc:borland:dslocal:";
	  //private static final String FILE = "e:/Work/jsp/java-database/try.jds";
	  private String FILE = "e:/Work/jsp/java-database/try.jds";
	 */
	private Connection connection;
	private PreparedStatement getStmt;
	private PreparedStatement putStmt;
	private PreparedStatement remStmt;
	private PreparedStatement updStmt;
	private PreparedStatement getAllStmt;

	public static UserRepository getInstance () throws UserRepositoryException {
		if (instance==null) {
			instance = new UserRepository ();
		}
		return instance;
	}


	private UserRepository () throws UserRepositoryException {
		String get = "SELECT * FROM users WHERE USERID=?";
		String put = "INSERT INTO users VALUES ( ?,?,?,?,?,? )";
		String rem = "DELETE FROM users WHERE USERID=?";
		String upd = "UPDATE users SET PASSWD=?, USERNAME=?, TYPE=?, EMAIL=?, TELEPHONE=? WHERE USERID=?";
		String all = "SELECT * FROM users ORDER BY USERID";

		// get the file name of the database
		XGSPConfiguration xgsp_conf = XGSPConfiguration.getInstance ();
		//if (! (xgsp_conf.getDataBaseURL()).equals("")) {
		//dbURL = xgsp_conf.getDataBaseURL();
		//}
		///if (! (xgsp_conf.getDataBaseName()).equals("")) {
		//FILE = xgsp_conf.getDataBaseName();
		//}
		try {
			//   logger.info ("user rep connection ...");
			DBConnection db = new DBConnection ();
			connection = db.getConnection ();

			if (connection==null) {
				//      logger.info ("null !");

			}

			getStmt = connection.prepareStatement (get);
			putStmt = connection.prepareStatement (put);
			remStmt = connection.prepareStatement (rem);
			updStmt = connection.prepareStatement (upd);
			getAllStmt = connection.prepareStatement (all);
		} catch (ClassNotFoundException e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			throw new UserRepositoryException ("No Driver Available!");
		} catch (SQLException e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			throw new UserRepositoryException (e.getMessage ());
		} catch (Exception e) {
//         logger.severe ("Exception : "+this.getClass ().getName ());
			//       logger.severe ("Exception : "+e.getMessage ());
			//     logger.severe ("Exception : "+e.toString ());

			e.printStackTrace ();
		}
	}


	private UserBean makeUser (ResultSet results) throws UserRepositoryException {
		try {
			UserBean user = new UserBean ();
			user.setUserid (results.getString ("USERID"));
			user.setPasswd (results.getString ("PASSWD"));
			user.setUsername (results.getString ("USERNAME"));
			user.setUsertype (results.getString ("TYPE"));
			user.setEmailaddr (results.getString ("EMAIL"));
			user.setTelephone (results.getString ("TELEPHONE"));

			return user;
		} catch (SQLException e) {
			throw new UserRepositoryException (e.getMessage ());
		}
	}


	public UserBean getUser (String userid) throws UnknownUserException,
			 UserRepositoryException {
		try {
			ResultSet results;
			synchronized (getStmt) {
				getStmt.clearParameters ();
				getStmt.setString (1, userid);
				results = getStmt.executeQuery ();
			}
			if (results.next ()) {
				return makeUser (results);
			} else {
				throw new UnknownUserException ("could not find User"+userid);
			}
		} catch (SQLException e) {
			throw new UserRepositoryException (e.getMessage ());
		}
	}


	// Get all User records
	public UserBean[] getUsers () throws UserRepositoryException {
		try {
			ResultSet results;
			Collection users = new ArrayList ();
			synchronized (getAllStmt) {
				results = getAllStmt.executeQuery ();
			}
			UserBean user;
			while (results.next ()) {
				users.add (makeUser (results));
			}
			return (UserBean[]) users.toArray (new UserBean[0]);
		} catch (SQLException e) {
			throw new UserRepositoryException (e.getMessage ());
		}
	}


	public void put (UserBean user) throws
			 UserRepositoryException {
		try {
			synchronized (putStmt) {
				putStmt.clearParameters ();
				putStmt.setString (1, user.getUserid ());
				putStmt.setString (2, user.getPasswd ());
				putStmt.setString (3, user.getUsername ());
				putStmt.setString (4, user.getUsertype ());
				putStmt.setString (5, user.getEmailaddr ());
				putStmt.setString (6, user.getTelephone ());
				putStmt.executeUpdate ();
			}
		} catch (SQLException e) {
			throw new UserRepositoryException (e.getMessage ());
		}
	}


	// delete a User record
	public void removeUser (String userid) throws
			 UserRepositoryException {
		try {
			synchronized (remStmt) {
				remStmt.clearParameters ();
				remStmt.setString (1, userid);
				int rowsChanged = remStmt.executeUpdate ();
				if (rowsChanged<1) {
					throw new UnknownUserException ("Can't delete User# "+userid);
				}
			}
		} catch (SQLException e) {
			throw new UserRepositoryException (e.getMessage ());
		}
	}


	// Update the old User record
	public void update (UserBean user) throws
			 UnknownUserException, UserRepositoryException {
		try {
			synchronized (updStmt) {
				updStmt.clearParameters ();
				updStmt.setString (6, user.getUserid ());
				updStmt.setString (1, user.getPasswd ());
				updStmt.setString (2, user.getUsername ());
				updStmt.setString (3, user.getUsertype ());
				updStmt.setString (4, user.getEmailaddr ());
				updStmt.setString (5, user.getTelephone ());
				int rowsChanged = updStmt.executeUpdate ();
				if (rowsChanged<1) {
					throw new UnknownUserException ("Could not find User# "+
							 user.getUserid ());
				}
			}
		} catch (SQLException e) {
			throw new UserRepositoryException (e.getMessage ());
		}
	}
}
