/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

import java.util.GregorianCalendar;
import java.util.*;
import java.sql.Timestamp;


public class Utility {
	private Utility () {
	}


	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	public static boolean isTimeConflict (SessionBean session,
								   SessionRepository sesRep) throws
			 SessionConflictOrImpossiblity, SessionRepositoryException {
		SessionBean[] sbs = sesRep.getSessions ();
		Collection sessions = new ArrayList ();
		SessionBean sb;

		System.out.println ("\n\n\nchecking --> SessionBean ? SessionRepository");
		for (int i = 0; i<sbs.length; i++) {
			System.out.println ("\nchecking ("+i+
							") --> SessionBean(session) ? SessionBean(sb), from SessionRepository");

			sb = sbs[i];

			if (Utility.isTimeConflict (session, sb)) {
				System.out.println ("time conflict found ("+i+
								") --> SessionBean(session) ? SessionBean(sb), from SessionRepository");
				System.out.println ("sb.getSessionstart().toString()      : "+
								sb.getSessionstart ().toString ());
				System.out.println ("sb.getSessionend().toString()        : "+
								sb.getSessionend ().toString ());
				System.out.println ("\n");
				//
				System.out.println ("session.getSessionstart().toString() : "+
								session.getSessionstart ().toString ());
				System.out.println ("session.getSessionend().toString()   : "+
								session.getSessionend ().toString ());
				System.out.println ("\n\n");

				return true;
			}
		}
		return false;
	}


	public static boolean isSameRoom (SessionBean A, SessionBean B) {
		if (A.getSessionplace ().equalsIgnoreCase (B.getSessionplace ())) {
			return true;
		}
		return false;
	}


	public static boolean isTimeConflict (SessionBean A, SessionBean B) {

		if (!Utility.isSameRoom (A, B)) {
			return false;
		}

		//case:1 --> A < B, but overlap
		if (Utility.isAOverlapBeforeOrEqualToB (A, B)) {
			System.out.println ("isTimeConflictBetweenAAndB : pass ... ");
			return true;
		}

		//case:2 --> overlap
		if (Utility.isAContainsOrEqualToB (A, B)) {
			System.out.println ("isTimeConflictBetweenAAndB : pass ... ");
			return true;
		}

		//case:3 --> B < A, but overlap
		if (Utility.isAOverlapAfterOrEqualToB (A, B)) {
			System.out.println ("isTimeConflictBetweenAAndB : pass ... ");
			return true;
		}

		System.out.println ("isTimeConflictBetweenAAndB : "+false);
		return false;
	}


	public static boolean isAOverlapBeforeOrEqualToB (SessionBean sA,
			 SessionBean sB) {
		//case:1 --> sA < sB, but overlap
		// A_start <= B_start < A_stop

		Timestamp A_start = sA.getSessionstart ();
		Timestamp B_start = sB.getSessionstart ();
		Timestamp A_end = sA.getSessionend ();

		if (A_start.before (B_start)||A_start.equals (B_start)) {
			if (B_start.before (A_end)||B_start.equals (A_end)) {
				System.out.println ("isAOverlapBeforeOrEqualToB : "+true);
				return true;
			}
		}
		System.out.println ("isAOverlapBeforeOrEqualToB : "+false);
		return false;
	}


	public static boolean isAContainsOrEqualToB (SessionBean A, SessionBean B) {
		//case:2 --> overlap

		Timestamp A_start = A.getSessionstart ();
		Timestamp B_start = B.getSessionstart ();
		Timestamp A_end = A.getSessionend ();

		if (A_start.before (B_start)||A_start.equals (B_start)) {
			if (B_start.before (A_end)||B_start.equals (A_end)) {
				System.out.println ("isAContainsOrEqualToB : "+true);
				return true;
			}
		}
		System.out.println ("isAContainsOrEqualToB : "+false);
		return false;
	}


	public static boolean isAOverlapAfterOrEqualToB (SessionBean A, SessionBean B) {
		//case:3 --> sB < sA, but overlap

		Timestamp A_start = A.getSessionstart ();
		Timestamp B_start = B.getSessionstart ();
		Timestamp A_end = A.getSessionend ();

		if (A_start.before (B_start)||A_start.equals (B_start)) {
			if (B_start.before (A_end)||B_start.equals (A_end)) {
				System.out.println ("isAOverlapAfterOrEqualToB : "+true);
				return true;
			}
		}
		System.out.println ("isAOverlapAfterOrEqualToB : "+false);
		return false;
	}


	public static boolean isSameDay (Calendar aa, Calendar bb) {
		Calendar a = (Calendar) aa.clone ();
		Calendar b = (Calendar) bb.clone ();

		int aDay = a.get (Calendar.DAY_OF_MONTH);
		int bDay = b.get (Calendar.DAY_OF_MONTH);
		int aMonth = a.get (Calendar.MONTH);
		int bMonth = b.get (Calendar.MONTH);
		int aYear = a.get (Calendar.YEAR);
		int bYear = b.get (Calendar.YEAR);

		if (aYear==bYear) {
			if (aMonth==bMonth) {
				if (aDay==bDay) {
					return true;
				}
			}
		}
		return false;
	}
}
