/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver;

import java.lang.*;
import java.util.*;


/**
 *  This class keeps the initialization configuration attributes
 */
public class XGSPConfiguration {
	private static XGSPConfiguration instance;
	private java.util.logging.Logger logger = java.util.logging.Logger.getLogger (this.
			 getClass ().getName ());

	// Server's Address: H323Gateway, SIPGateway, AVSessionServer
	private String H323GW_Addr;
	private String SIPGW_Addr;
	private String SessionSv_Addr;
	// RealStreaming Server's Address
	private String HelixSvr_Addr;
	private String HelixSvr_Port; // RTSP port
	private String RealGW_Addr;
	// NaradaBroker Address & Port
	private String NaradaBroker_Addr;
	private String NaradaBroker_Port;
	// MetaData Database URL
	private String Database_URL;
	private String App_database_name;

	private boolean stream_monitor_thread;

	// Server's State
	private boolean[] ServerState = new boolean[7];
	private Hashtable ServerNameMapping;

	public static XGSPConfiguration getInstance () {
		if (instance==null) {
			instance = new XGSPConfiguration ();
		}
		return instance;
	}


	public XGSPConfiguration () {
		this.H323GW_Addr = "";
		this.SessionSv_Addr = "";

		this.Database_URL = "";
		this.App_database_name = "";
		this.stream_monitor_thread = false;

		for (int i = 0; i<7; i++) {
			ServerState[i] = false;
		}
		ServerNameMapping = new Hashtable ();
		ServerNameMapping.put ("H323Gateway", new Integer (0));
		ServerNameMapping.put ("H323Gatekeeper", new Integer (1));
		ServerNameMapping.put ("SessionServer", new Integer (2));
		ServerNameMapping.put ("AudioServer", new Integer (3));
		ServerNameMapping.put ("VideoServer", new Integer (4));
		ServerNameMapping.put ("SnapshotServer", new Integer (5));
		ServerNameMapping.put ("WebServer", new Integer (6));
	}


	// H323Gateway
	public void setH323GW (String gw_addr) {
		this.H323GW_Addr = gw_addr;
	}


	public String getH323GW () {
		return H323GW_Addr;
	}


	// SIPGateway
	public void setSIPGW (String gw_addr) {
		this.SIPGW_Addr = gw_addr;
	}


	public String getSIPGW () {
		return SIPGW_Addr;
	}


	// AV-Session-Server
	public void setSessionSv (String sv_addr) {
		this.SessionSv_Addr = sv_addr;
	}


	public String getSessionSv () {
		return SessionSv_Addr;
	}


	// Helix Server
	public void setHelixSvr (String sv_addr) {
		this.HelixSvr_Addr = sv_addr;
	}


	public String getHelixSvr () {
		return this.HelixSvr_Addr;
	}


	public void setHelixPort (String sv_port) {
		this.HelixSvr_Port = sv_port;
	}


	public String getHelixPort () {
		return this.HelixSvr_Port;
	}


	// Real Streaming Gateway
	public void setRealGW (String gw_addr) {
		this.RealGW_Addr = gw_addr;
	}


	public String getRealGW () {
		return this.RealGW_Addr;
	}


	// NaradaBroker
	public String getNBAddr () {
		return this.NaradaBroker_Addr;
	}


	public void setNBAddr (String NBAddr) {
		this.NaradaBroker_Addr = NBAddr;
	}


	public String getNBPort () {
		return this.NaradaBroker_Port;
	}


	public void setNBPort (String NBPort) {
		this.NaradaBroker_Port = NBPort;
	}


	// DataBase
	public void setDataBaseURL (String database_url) {
		this.Database_URL = database_url;
	}


	public String getDataBaseURL () {
		return (this.Database_URL);
	}


	public void setDataBaseName (String database_addr) {
		this.App_database_name = database_addr;
	}


	public String getDataBaseName () {
		return App_database_name;
	}


	////////////////////////////////////////////////////////////////////////////
	////////////// Server Status Monitor
	////////////////////////////////////////////////////////////////////////////
	public boolean getStreamMonStatus () {
		return this.stream_monitor_thread;
	}


	public void setStreamMonStatus (boolean flag) {
		this.stream_monitor_thread = flag;
	}


	public void setServerState (String server_name, boolean state) {
		int server_index;
		if (ServerNameMapping.containsKey (server_name)) {
			server_index = ((Integer) (ServerNameMapping.get (server_name))).intValue ();
			this.ServerState[server_index] = state;
		}
	}


	public boolean getServerState (String server_name) {
		int server_index;
		if (ServerNameMapping.containsKey (server_name)) {
			server_index = ((Integer) (ServerNameMapping.get (server_name))).intValue ();
			return this.ServerState[server_index];
		} else {
			return false;
		}
	}


	public int getServerIndex (String server_name) {
		int server_index;
		if (ServerNameMapping.containsKey (server_name)) {
			server_index = ((Integer) (ServerNameMapping.get (server_name))).intValue ();
			return server_index;
		} else {
			return (-1);
		}
	}


	public String[] getServerName () {
		int i;
		String[] all_server_name = new String[7];
		Enumeration server_list = ServerNameMapping.keys ();

		while (server_list.hasMoreElements ()) {
			String tmp = (String) server_list.nextElement ();
			i = ((Integer) (ServerNameMapping.get (tmp))).intValue ();
			all_server_name[i] = tmp;
		}
		return all_server_name;
	}


	public boolean[] getAllServerState () {
		return ServerState;
	}
}