/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.admin.meetingMgt;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import java.sql.Timestamp;

import cgl.globalmmcs.webserver.*;


public class GetAllSession implements Command {
	private String next;
	public GetAllSession (String next) {
		this.next = next;
	}


	public String execute (HttpServletRequest req) throws CommandException {
		try {
			SessionRepository sessions = SessionRepository.getInstance ();
			//SessionBean[] SessionList = sessions.getSessions();
			SessionBean[] SessionList = querySession (req);
			req.setAttribute ("sessions", SessionList);
			return next;
		} catch (SessionRepositoryException ce) {
			throw new CommandException ("Get Session command:"+ce.getMessage ());
		}
	}


	/**
	 * combo query for session repository
	 * (1) query by date
	 * (2) query by combo parameters
	 * (3) return all the sessions in the database
	 */
	public SessionBean[] querySession (HttpServletRequest req) throws
			 SessionRepositoryException {
		HttpSession ses = req.getSession (false);
		// read the day parameters
		String daystring = req.getParameter ("daylong");
		if (daystring!=null) {
			System.out.println ("daystring, (daylong)   : "+daystring);
		}
		// read the user information for this request
		Object o = null;
		String userid = null;

		if (((Object) ses.getValue ("userid"))!=null) {
			o = (Object) ses.getValue ("userid");
			if (o!=null) { // harun
				userid = (String) o;
				System.out.println ("userid : "+userid);
			}
		}
		/** build a query parameter list
		 * < day, dayPass_from, dayPass_to, clock time, owner, room >
		 */
		SessionParameters sp = new SessionParameters ();
		// set day parameter
		if (daystring!=null) {
			sp.setDaystring (daystring);
		}
		// set day pass parameter ( < dayPass_from, dayPass_to > )
		String dayPassString_from = null;
		dayPassString_from = req.getParameter ("dayPassString_from");
		if (dayPassString_from!=null) {
			dayPassString_from.trim ();
		}
		if ((dayPassString_from!=null)&&
		    (!dayPassString_from.equalsIgnoreCase (""))) {
			sp.setDayPassString_from (dayPassString_from);
		}
		//-------------------------------------------------------------------
		String dayPassString_to = null;
		dayPassString_to = req.getParameter ("dayPassString_to");
		if (dayPassString_from!=null) {
			dayPassString_from.trim ();
		}
		if ((dayPassString_to!=null)&&(!dayPassString_to.equalsIgnoreCase (""))) {
			sp.setDayPassString_to (dayPassString_to);
		}
		// set the time parameter
		String time = null;
		time = req.getParameter ("time");
		if (time!=null) {
			sp.setTime (time);
		}
		// set the owner parameter
		String whosemeetings = null;
		whosemeetings = req.getParameter ("whosemeetings");
		if (whosemeetings!=null) {
			if (whosemeetings.equalsIgnoreCase ("my")) {
				sp.setSessionowner (userid);
			}
		}
		// set the room parameter
		String room = null;
		room = req.getParameter ("room");
		if (room!=null) {
			sp.setRoom (room);
		}
		// make the query and return result
		SessionRepository sessions = SessionRepository.getInstance ();
		SessionBean[] SessionList = null;

		// case # 1: query by a day
		if (daystring!=null) {
			SessionList = sessions.getSessionsInASpecificDay (daystring);
			return (SessionList);
		}
		// case # 2: query by the SessionParameter
		// Wenjun: here we should not use Exception to make different case!!
		try {
			SessionList = sessions.getSessions (sp);
			return (SessionList);

		} catch (Exception e) {
			e.printStackTrace ();
		}
		// case # 3: simple return all the session
		SessionList = sessions.getSessions ();
		return (SessionList);
	}


	/**
	 * put all the dates of the sessions into an Array
	 */
	public Vector getAllSessionDatesInVectorOfCalendars () {
		Vector v = new Vector ();
		try {
			SessionRepository sessions = SessionRepository.getInstance ();
			SessionBean[] SessionList = sessions.getSessions ();

			Calendar c;
			SessionBean sb;
			Timestamp ts;
			long g;

			int length = SessionList.length;

			for (int i = 0; i<length; i++) {
				sb = SessionList[i];
				ts = sb.getSessionstart ();
				g = ts.getTime ();
				c = new GregorianCalendar ();
				c.setTimeInMillis (g);
				v.addElement (c);
			}

		} catch (SessionRepositoryException ce) {
			try {
				throw new CommandException ("Get Session command:"+ce.getMessage ());
			} catch (CommandException ex) {
			}
		}
		return v;
	}
}