/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.admin.meetingMgt;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import java.sql.*;

import cgl.globalmmcs.webserver.*;
import cgl.globalmmcs.webserver.sessionmgt.*;


/**
 * This servelt updates the information of the existing session
 */
public class editSession implements Command {
	private String next;

	public editSession (String next) {
		this.next = next;
	}


	/**
	 * any active session has been locked and can't be modified
	 */
	public String execute (HttpServletRequest req) throws CommandException {
		try {
			SessionRepository sessions = SessionRepository.getInstance ();
			XGSPAVSessionManager manager = XGSPAVSessionManager.getInstance ();

			// put the session info into the existing data record
			SessionBean sess = new SessionBean ();

			String sessionid = req.getParameter ("sessionid");
			String session_name = req.getParameter ("sessionname");
			String session_owner = req.getParameter ("sessionowner");
			String session_chair = req.getParameter ("sessionchair");
			String session_des = req.getParameter ("sessiondes");

			// Wenjun Wu: here we need to check some conflict just like AddSession
			//Timestamp session_start = Timestamp.valueOf(req.getParameter("sessionstart"));
			//Timestamp session_end = Timestamp.valueOf(req.getParameter("sessionend"));
			String string;
			java.util.Date date;
			long longDate;
			Timestamp session_start, session_end;
			try {
				string = req.getParameter ("sessionstart");
				date = new java.util.Date (string);
				longDate = date.getTime ();
				session_start = new Timestamp (longDate);
			} catch (java.lang.IllegalArgumentException e) {
				session_start = Timestamp.valueOf (req.getParameter ("sessionstart"));
			}
			try {
				string = req.getParameter ("sessionend");
				date = new java.util.Date (string);
				longDate = date.getTime ();
				session_end = new Timestamp (longDate);
			} catch (java.lang.IllegalArgumentException e) {
				session_end = Timestamp.valueOf (req.getParameter ("sessionend"));
			}

			String session_place = req.getParameter ("sessionplace");

			sess.setSessionid (sessionid);
			sess.setSessionname (session_name);
			sess.setSessionowner (session_owner);
			sess.setSessionchair (session_chair);
			sess.setSessiondes (session_des);
			sess.setSessionstart (session_start);
			sess.setSessionend (session_end);
			sess.setSessionplace (session_place);

			synchronized (manager) {
				sessions.update (sess);
				// if the date of this session is today, it should be added into waitList in XGSPAVSessionManager
				// if the date of this session isn't today, it should be removed from the list
				manager.editSession (sess);
			}
			// Generate the whole list of the community
			SessionBean[] SessionList = sessions.getSessions ();
			req.setAttribute ("sessions", SessionList);
			return next;
		} catch (SessionRepositoryException fe) {
			throw new CommandException ("Edit Session Command: "+fe.getMessage ());
		}
	}
}