/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.admin.meetingMgt;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import java.sql.*;

import cgl.globalmmcs.webserver.*;
import cgl.globalmmcs.webserver.sessionmgt.*;


/**
 * This commands collects the sessions that are ready for joining
 * the following sessions are availible: Permanent Session,  Today Session & Activated Session
 */
public class list_today_session implements Command {
	private String next;
	public list_today_session (String next) {
		this.next = next;
	}


	public String execute (HttpServletRequest req) throws CommandException {
		try {
			// fetch all the sessions whose start time is after current time
			SessionRepository sessions = SessionRepository.getInstance ();
			java.util.Date now = new java.util.Date ();
			long mills = now.getTime ();
			Timestamp datebefore = new Timestamp (mills);
			mills += 24*3600*1000;
			Timestamp dateafter = new Timestamp (mills);
			SessionBean[] tmpSess = sessions.findSessions (dateafter, datebefore);
			// create Active-Session-List
			XGSPAVSessionManager manager = XGSPAVSessionManager.getInstance ();
			ActiveSession[] activeSSS = manager.getActiveSessions ();
			ArrayList activeS = new ArrayList ();
			for (int i = 0; i<activeSSS.length; i++) {
				SessionBean tmp = activeSSS[i].get_sessioninfo ();
				activeS.add (tmp);
			}
			// create Waiting-Session-List
			ArrayList waitS = manager.getWaitList ();
			for (int i = 0; i<tmpSess.length; i++) {
				SessionBean tmp = tmpSess[i];
				boolean flag1 = false;
				boolean flag2 = false;
				String tmp_sessid = tmp.getSessionid ();
				for (int j = 0; j<waitS.size (); j++) {
					SessionBean ttmp = (SessionBean) waitS.get (j);
					if (tmp_sessid.equals (ttmp.getSessionid ())) {
						flag1 = true;
						break;
					}
				}
				for (int j = 0; j<activeS.size (); j++) {
					SessionBean ttmp = (SessionBean) activeS.get (j);
					if (tmp_sessid.equals (ttmp.getSessionid ())) {
						flag2 = true;
						break;
					}
				}
				// if the temp session is not included the waiting-list and active-list, then include it
				if (!flag1&&!flag2) {
					manager.addWaitList (tmp);
				}
			}
			waitS = manager.getWaitList ();
			req.setAttribute ("active_sess", activeS);
			req.setAttribute ("wait_sess", waitS);

			return next;
		} catch (SessionRepositoryException ce) {
			throw new CommandException ("Get Session command:"+ce.getMessage ());
		}
	}
}