/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.admin.roomMgt;

/**
 * <p>Title: editRoom </p>
 * <p>Description: XGSP Conference Web Server</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Community Grids Lab, Indiana University</p>
 * @author Wenjun Wu, Harun Altay
 * @version 1.0
 */

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import cgl.globalmmcs.webserver.*;
import cgl.globalmmcs.webserver.sscomm.MediaDescription;
import cgl.globalmmcs.webserver.sscomm.SessionMediaDescription;


/**
 *  change the profile of the room
 *  (1) synchronized the related information with AVSession Server
 *  (2) not allowed to change the profile if the room is used by current conferences.
 */
public class editRoom implements Command {
	private String next;

	public editRoom (String next) {
		this.next = next;
	}


	public String execute (HttpServletRequest req) throws CommandException {
		try {
			RoomRepository rooms = RoomRepository.getInstance ();
			RoomBean room = new RoomBean ();

			String roomid = req.getParameter ("roomid");
			String communityid = req.getParameter ("communityid");
			String roomname = req.getParameter ("roomname");
			String roomtype = req.getParameter ("roomtype");
			// can't get the addr XML directly from the web page
			String AIPAddr = req.getParameter ("AIPAddr");
			String APort = req.getParameter ("AIPPort").trim ();
			String AudioFmt = req.getParameter ("AMediaFormat");
			String VIPAddr = req.getParameter ("VIPAddr");
			String VPort = req.getParameter ("VIPPort").trim ();
			String VideoFmt = req.getParameter ("VMediaFormat");

			MediaDescription audioDes = new MediaDescription ();
			MediaDescription videoDes = new MediaDescription ();
			audioDes.setMediaType ("AUDIO");
			audioDes.setAddr (AIPAddr);
			audioDes.setPort ((new Integer (APort)).intValue ());
			audioDes.setCodecType (AudioFmt);
			videoDes.setMediaType ("VIDEO");
			videoDes.setAddr (VIPAddr);
			videoDes.setPort ((new Integer (VPort)).intValue ());
			videoDes.setCodecType (VideoFmt);

			//String roomaddr = req.getParameter("roomaddr");
			SessionMediaDescription sessMDes = new SessionMediaDescription ();
			sessMDes.setAudioDesc (audioDes);
			sessMDes.setVideoDesc (videoDes);
			sessMDes.setSessionID (roomid);
			String roomaddr = sessMDes.toXML ("");

			room.setRoomid (roomid);
			room.setCommunityid (communityid);
			room.setRoomname (roomname);
			room.setRoomtype (roomtype);
			room.setRoomaddr (roomaddr);

			rooms.update (room);

			// Generate the whole list of the community
			RoomBean[] roomList = rooms.getRooms ();
			req.setAttribute ("rooms", roomList);
			return next;
		}
		/** catch (UnknownUserException uf ){
		 //throw new CommandException("GetCommand: " + uf.getMessage());
		 return "error.jsp";
		 }**/
		catch (RoomRepositoryException fe) {
			throw new CommandException ("GetCommand: "+fe.getMessage ());
		}
	}
}