/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.sessionmgt;

import java.lang.*;
import java.util.*;
import java.util.logging.*;

import cgl.globalmmcs.webserver.*;
import cgl.globalmmcs.webserver.sscomm.*;


public class ActiveSession {
	private String session_id;
	private SessionBean session_info;

	private int cur_participants;
	private int old_participants;

	private Hashtable member_list; // userid --> key
	private Hashtable stream_list; // user_name --> key
	private Hashtable all_stream_list; // ssrc --> key

	private String[] mixed_stream_list; // video mixing stream ( ssrc )

	private Hashtable VideoMixerList;

	int MixerCounter;

	private static Logger logger = Logger.getLogger (ActiveSession.class.getName ());

	public ActiveSession (String session_id) {
		this.session_id = session_id;
		this.member_list = new Hashtable ();
		this.stream_list = new Hashtable ();
		this.all_stream_list = new Hashtable ();

		this.VideoMixerList = new Hashtable ();
		this.MixerCounter = 0;
		// get session_info
		try {
			SessionRepository sessions = SessionRepository.getInstance ();
			session_info = sessions.getSession (session_id);
		} catch (Exception fe) {
			fe.printStackTrace ();
		}
	}


	public ActiveSession (SessionBean sess) {
		this.session_id = sess.getSessionid ();
		this.session_info = sess;

		this.member_list = new Hashtable ();
		this.stream_list = new Hashtable ();
		this.all_stream_list = new Hashtable ();

		this.VideoMixerList = new Hashtable ();
		this.MixerCounter = 0;
	}


	public String get_sessionid () {
		return this.session_id;
	}


	public SessionBean get_sessioninfo () {
		return this.session_info;
	}


	//////////////////////////////////////////////////////////////////////////
	/// MemberShip methods
	//////////////////////////////////////////////////////////////////////////
	// methods on the member list in this session
	public void add_member (MemberBean mem) {
		member_list.put (mem.getUserid (), mem);
	}


	public void rem_member (MemberBean mem) {
		member_list.remove (mem.getUserid ());
	}


	public void rem_member (String userid) {
		member_list.remove (userid);
	}


	public MemberBean[] getMembers () {
		String tmp_user_id;
		Collection members = new ArrayList ();
		Enumeration keylist = member_list.keys ();
		while (keylist.hasMoreElements ()) {
			tmp_user_id = (String) keylist.nextElement ();
			members.add (member_list.get (tmp_user_id));
		}
		return (MemberBean[]) members.toArray (new MemberBean[0]);
	}


	public MemberBean getMember (String userid) {
		return (MemberBean) member_list.get (userid);
	}


	public boolean hasMember (String userid) {
		return member_list.containsKey (userid);
	}


	public int cntMember () {
		return member_list.size ();
	}


	//////////////////////////////////////////////////////////////////////////////
	///// Stream methods
	//////////////////////////////////////////////////////////////////////////////
	// methods on the stream list in this session
	public void SetStreamList (Vector streams) {
		int new_len;
		// firstly remove all the old streams
		if (all_stream_list!=null) {
			all_stream_list.clear ();
			stream_list.clear ();
		}
		for (int i = 0; i<streams.size (); i++) {
			StreamBean tmpstream = (StreamBean) streams.elementAt (i);
			addStream (tmpstream);
		}
	}


	public StreamBean[][] GetStreamList () {
		String tmp_username;
		Collection streams = new ArrayList ();
		// get all the userNames
		Enumeration keylist = stream_list.keys ();
		while (keylist.hasMoreElements ()) {
			tmp_username = (String) keylist.nextElement ();
			// we get StreamBean[] here
			ArrayList tmp_streams = (ArrayList) stream_list.get (tmp_username);
			streams.add ((StreamBean[]) (tmp_streams.toArray (new StreamBean[0])));
		}
		return (StreamBean[][]) streams.toArray (new StreamBean[0][0]);
	}


	public void addStream (StreamBean newStream) {
		// put the stream into the list of SSRC
		all_stream_list.put (newStream.getSsrc (), newStream);
		// put the stream into the list of username
		ArrayList old_streams = (ArrayList) stream_list.get (newStream.getUsername ());
		if (old_streams!=null) {
			stream_list.remove (old_streams);
			old_streams.add (newStream);
			stream_list.put (newStream.getUsername (), old_streams);
		} else {
			ArrayList new_streams = new ArrayList ();
			new_streams.add (newStream);
			stream_list.put (newStream.getUsername (), new_streams);
		}
		/** streamBean [] old_streams = (StreamBean[] )stream_list.get( newStream.getUsername() );
		   if ( old_streams != null ){
		  new_len = old_streams.length;
		   } else
		  new_len = 0;
		   StreamBean[]  new_streams = new StreamBean[new_len+1];
		   if ( new_len > 0 )
		 System.arraycopy( old_streams, 0, new_streams, 0, old_streams.length );
		   new_streams[new_len] = newStream;
		   stream_list.put( newStream.getUsername(), new_streams );
		 **/
	}


	/**
	 * When a ByeStreamEvent happens on the stream, this stream has to be removed
	 * @ oldStream: with EventType ByeStreamEvent
	 */
	public void remStream (StreamBean oldStream) {
		// remove the stream from the SSRC list
		String ssrc = oldStream.getSsrc ();
		String userName = oldStream.getUsername ();
		// get the olderStream: assume it's EventType is NewStreamEvent!!
		StreamBean olderStrm = (StreamBean) all_stream_list.get (ssrc);
		all_stream_list.remove (ssrc);
		// use the older user name for safety
		// Wenjun: we found NULLPointerException for this. Date: 04/22/2004
		if (!userName.equals (olderStrm.getUsername ())) {
			userName = olderStrm.getUsername ();
		}
		// remove the stream from the UserName list
		ArrayList old_streams = (ArrayList) stream_list.get (oldStream.getUsername ());
		// if there is no such userName, search all the list to find it
		if (old_streams!=null) {
			stream_list.remove (old_streams);
			old_streams.remove (olderStrm);
			stream_list.put (oldStream.getUsername (), old_streams);
		} else {
			logger.info ("stream userName is werid: "+ssrc+":"+
					   oldStream.getUsername ()+":"+olderStrm.getUsername ());
		}

	}


	public StreamBean getStream (String ssrc) {
		return (StreamBean) (all_stream_list.get (ssrc)); //???
	}


	/**
	 * When a ActiveToPassive Event arrives, disable this stream in the session
	 */
	public void disableStream (StreamBean passiveAV) {
		String ssrc = passiveAV.getSsrc ();
		StreamBean tmp = (StreamBean) all_stream_list.get (ssrc);
		if (tmp!=null) {
			tmp.setActive (false);
		}
	}


	/**
	 * When a PassiveToActive Event arrives, enalbe this stream again in the session
	 * @return
	 */
	public void enableStream (StreamBean activeAV) {
		String ssrc = activeAV.getSsrc ();
		StreamBean tmp = (StreamBean) all_stream_list.get (ssrc);
		if (tmp!=null) {
			tmp.setActive (true);
		}
	}


	///////////////////////////////////////////////////////////////////////
	/////// Stream Mixer Methods
	///////////////////////////////////////////////////////////////////////
	public int GetVideoMixerCnt () {
		return this.MixerCounter;
	}


	public void AddVideoMixer (VideoMixer newMixer) {
		String MixerID = newMixer.getMixerID ();
		VideoMixerList.put (MixerID, newMixer);
		MixerCounter++;
	}


	public void RemVideoMixer (String MixerID) {
		VideoMixerList.remove (MixerID);
		//MixerCounter--;
	}


	public VideoMixer[] getAllMixer () {
		Collection mixers = new ArrayList ();
		Enumeration keylist = VideoMixerList.keys ();
		while (keylist.hasMoreElements ()) {
			String tmp_mixerID = (String) keylist.nextElement ();
			VideoMixer tmp_mixer = (VideoMixer) VideoMixerList.get (tmp_mixerID);
			mixers.add (tmp_mixer);

		}
		return (VideoMixer[]) mixers.toArray (new VideoMixer[0]);
	}


	public void SetMixStream (String[] mixed_video) {
		this.mixed_stream_list = mixed_video;
	}


	public VideoMixer getMixer (String mixerID) {
		VideoMixer tmpMixer = (VideoMixer) VideoMixerList.get (mixerID);
		return tmpMixer;
	}


	//public String[] GetMixStream() {
	//	return this.mixed_stream_list;
	//}
	// a general string arrary compare alogrithm; false: unchanged or new_src is null; true: new
	public boolean CompareSelSrc (String[] new_src, String[] old_src) {
		boolean tmp_flag;
		if (new_src==null) {
			return false;
		}
		if (old_src==null) { // old_src == null && new_src != null
			return true;
		}
		for (int i = 0; i<new_src.length; i++) {
			tmp_flag = false;
			for (int j = 0; j<old_src.length; j++) {
				if (old_src[j].equals (new_src[i])) {
					tmp_flag = true;
					break;
				}
			}
			if (!tmp_flag) { // !old_src.contains(new_src[i]
				return true;
			}
		}
		// if new_src is smaller than old src, then it is also new
		if (new_src.length!=old_src.length) {
			return true;
		} else {
			return false;
		}
	}


	///////////////////////////////////////////////////////////////////
	//// update the session status
	//////////////////////////////////////////////////////////////////
	public void updateSessionStatus (SessionStatusReport rpt) {
		//this.cur_participants = rpt.getCurPart();
		//this.old_participants = rpt.getPastPart();
	}
}