/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.sessionmgt;

import java.io.*;
import java.util.*;
import java.sql.*;
import java.util.logging.*;

import org.apache.axis.AxisFault;
import cgl.globalmmcs.webserver.*;


/*** Demon is a background thread for activate/deactivate XGSP AV session
 *** It only deals with the sessions which have the policy of "starting session 15 mins before the starting time
 ***/
public class ActiveSessionDemon extends Thread {
	private static ActiveSessionDemon instance;
	//private SessionActiveList active_sess_list;

	//private AdminServerChannel admin_channel;

	private RoomRepository rooms;
	private XGSPConfiguration xgsp_conf;
	private XGSPAVSessionManager manager;

	private boolean running;

	private static Logger logger = Logger.getLogger (ActiveSessionDemon.class.
			 getName ());

	public ActiveSessionDemon (XGSPAVSessionManager manager) {

		xgsp_conf = XGSPConfiguration.getInstance ();
		this.manager = manager;
		running = true;
		this.start ();

	}


	/** public static ActiveSessionDemon getInstance() {
	  if ( instance == null ){
	 instance = new ActiveSessionDemon() ;
	  }
	  return instance;
	   } **/
	/**
	 * This method activates the permanent sessions
	 * A permanent session is specified in Session Database
	 */
	public void activePermanentSessions () {
		ArrayList permanentList = manager.getPermanentList ();
		for (int i = 0; i<permanentList.size (); i++) {
			// delete it from permanent list
			SessionBean tmp = (SessionBean) permanentList.get (i);
			permanentList.remove (i);
			// send the command to AVSession-Server & add it into the active-session list
			manager.ActivateSession (tmp);
		}
	}


	/**
	 * This method activates the normal sessions
	 * A normal session is activated 15 mins before its start time
	 */
	public void activeWaitSessions () {
		ArrayList waitList = manager.getWaitList ();
		for (int i = 0; i<waitList.size (); i++) {
			SessionBean tmp = (SessionBean) waitList.get (i);
			// if the starting time has reached, then activate this session ( 15 mins ago )
			Timestamp sess_start_time = tmp.getSessionstart ();
			long mills_start = sess_start_time.getTime ();
			// current time
			java.util.Date now = new java.util.Date ();
			long mills_now = now.getTime ();
			long period = (mills_start-mills_now)/(1000*60); // 1 min
			if (period<15) { // activate this session
				manager.ActivateSession (tmp);
				waitList.remove (i);
			}
		}
	}


	/**
	 * This method De-Activates the normal sessions
	 * A normal session is De-activated when its ending time has arrived
	 */
	public void deactiveSessions () {
		ActiveSession[] tmpList = manager.getActiveSessions ();
		for (int i = 0; i<tmpList.length; i++) {
			// get the end time of this session
			SessionBean tmp_sess = tmpList[i].get_sessioninfo ();
			Timestamp sess_end_time = tmp_sess.getSessionend ();
			// current time
			java.util.Date now = new java.util.Date ();
			long mills = now.getTime ();
			Timestamp cur_time = new Timestamp (mills);
			// When session end time has passed, then deactivate the session
			String sess_des = tmp_sess.getSessiondes ();
			if (!sess_des.equals ("PERMANENT")&&
			    (sess_end_time.getTime ()<cur_time.getTime ())) {
				manager.DeActivateSession (tmpList[i]);
			}
		}
	}


	/**
	 * This method closes all the active sessions when it is interrupted
	 * this implmented the "restart" function
	 */
	public void closeAllSessions () {
		// for all the active sessions, de-activate it
		ActiveSession[] tmpList = manager.getActiveSessions ();
		for (int i = 0; i<tmpList.length; i++) {
			manager.DeActivateSession (tmpList[i]);
		}
	}


	/**
	 * Thread running routine
	 */
	public void run () {
		//try {
		// for the first run, we should start these permanent room sessions
		activePermanentSessions ();
		while (true) {
			if (running) {
				synchronized (manager) { // this section must be locked
					// search for the possible session need to be started
					// check whether there is any new permanent sessions here ( check the list )
					activePermanentSessions ();
					activeWaitSessions ();
				}
				// search each session in Active-Session-List and deactive those whose ending time have reached
				deactiveSessions ();
			}
			try {
				// sleep 5 mins ( 1 min ~ 60s )	// sleep(300*1000);
				sleep (60*1000); // sleep 1 min
			} catch (Exception xx) {
				xx.printStackTrace ();
				// restart all the stuffs: deActivate-all-the-sessions
				closeAllSessions ();
			}
		} // end of while

	}


	/**
	 * disable the running of the demon
	 */
	public void disable () {
		this.running = false;
	}


	public void restart () {
		this.running = true;
	}
}