/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.sessionmgt;

import java.util.*;
import java.sql.*;
import org.jdom.*;

import cgl.globalmmcs.webserver.*;
import cgl.globalmmcs.webserver.sscomm.*;
import java.util.logging.*;


/**
 * <p>Title: </p>
 * <p>Description: XGSP Conference Web Server Manager</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Community Grids Lab, Indiana University</p>
 * @author Wenjun Wu
 * @version 1.0
 */
public class XGSPAVSessionManager {
	private static XGSPAVSessionManager instance;

	private SessionActiveList ActiveSessionList; // all the sessions that are active
	private ArrayList SessionWaitList; // all the sessions of today that are scheduled
	private ArrayList PermanentSessionList; // all the sessions have permanet room assaignments

	private ActiveSessionDemon SessionDemon;

	private AdminServerChannel AdminChannel;

	private static Logger logger = Logger.getLogger (XGSPAVSessionManager.class.
			 getName ());

	public static XGSPAVSessionManager getInstance () {
		if (instance==null) {
			instance = new XGSPAVSessionManager ();
		}
		return instance;

	}


	private XGSPAVSessionManager () {
		ActiveSessionList = new SessionActiveList ();
		SessionWaitList = new ArrayList ();
		PermanentSessionList = new ArrayList ();
		AdminChannel = new AdminServerChannel ();
		AdminChannel.setManager (this);
	}


	/**
	 * restart XGSPAVSessionManager
	 * assume SessionDemon & AdminServerChannel are all reset
	 */
	public void restart () {
		// old state should be cleaned

		// create new running environment
		ActiveSessionList.clear ();
		AdminChannel.restart ();
		RestartSessionDemon ();
	}


	/**
	 * start Session Scheduler
	 */
	public void StartSessionDemo () {
		initList ();
		SessionDemon = new ActiveSessionDemon (this);
	}


	/**
	 * stop the Running of SessionDemon, and DeActivate all the active sessions
	 */
	public void StopSessionDemon () {
		SessionDemon.disable ();
		SessionDemon.interrupt ();
	}


	public void RestartSessionDemon () {
		// must avoid duplicate sessions in permanent/waiting list
		// firstly clean all the lists
		ActiveSessionList.clear ();
		SessionWaitList.clear ();
		PermanentSessionList.clear ();
		initList ();
		SessionDemon.restart ();
	}


	public ActiveSession[] getActiveSessions () {
		return (ActiveSessionList.getSessions ());
	}


	public ActiveSession getActiveSession (String sessionID) {
		return (ActiveSessionList.getSession (sessionID));
	}


	public ArrayList getWaitList () {
		return this.SessionWaitList;
	}


	public ArrayList getPermanentList () {
		return this.PermanentSessionList;
	}


	/**
	 * initialize the Session List in the manager: Permanent Session list, Waiting Session list
	 */
	public void initList () {
		try {
			SessionRepository session_database = SessionRepository.getInstance ();
			//get all the permanant sessions
			// if session.des = "PERMANANT", then it is regarded as a permanent session
			SessionBean[] tmpSessions = session_database.getPersistSessions ();
			for (int i = 0; i<tmpSessions.length; i++) {
				PermanentSessionList.add (tmpSessions[i]);
			}
			//get all the waiting list session
			tmpSessions = getTodaySessions ();
			for (int i = 0; i<tmpSessions.length; i++) {
				SessionWaitList.add (tmpSessions[i]);
			}
		} catch (Exception e) {
			e.printStackTrace ();
		}
	}


	public SessionBean[] getTodaySessions () {
		SessionBean[] sess = null;
		try {
			SessionRepository sessions = SessionRepository.getInstance ();
			java.util.Date now = new java.util.Date ();
			long mills = now.getTime ();
			Timestamp datebefore = new Timestamp (mills);
			mills += 24*3600*1000;
			Timestamp dateafter = new Timestamp (mills);
			sess = sessions.findSessions (dateafter, datebefore);
		} catch (Exception ce) {
			ce.printStackTrace ();
		}
		return sess;
	}


	/**
	 * The methods are used by Web Servelets to add/rem/mod the session instances
	 * in PermanentSessionList &
	 */
	public void addWaitList (SessionBean newSess) {
		String tmp = newSess.getSessiondes ();
		if (tmp.equals ("PERMANENT")) {
			PermanentSessionList.add (newSess);
		} else {
			SessionWaitList.add (newSess);
		}
	}


	public void remWaitList (String sessionID) {
		for (int i = 0; i<PermanentSessionList.size (); i++) {
			SessionBean tmp = (SessionBean) PermanentSessionList.get (i);
			if (sessionID.equals (tmp.getSessionid ())) {
				PermanentSessionList.remove (tmp);
				return;
			}
		}
		for (int i = 0; i<SessionWaitList.size (); i++) {
			SessionBean tmp = (SessionBean) SessionWaitList.get (i);
			if (sessionID.equals (tmp.getSessionid ())) {
				SessionWaitList.remove (tmp);
				return;
			}
		}
	}


	public void remWaitList (SessionBean oldSess) {
		String tmp = oldSess.getSessiondes ();
		if (tmp.equals ("PERMANENT")) {
			PermanentSessionList.remove (oldSess);
		} else {
			SessionWaitList.remove (oldSess);
		}
	}


	/**
	 * called by editSession servelet: when an old session is modified, the permanent/waiting
	 * lists have to been updated
	 */
	public void editSession (SessionBean newSess) {
		boolean Permanent = false;
		boolean Today = false;

		String tmp = newSess.getSessiondes ();
		// check the date
		Timestamp session_start = newSess.getSessionstart ();
		java.util.Date now = new java.util.Date ();
		long now_mills = now.getTime ();
		long tomorrow_mills = now_mills+24*3600*1000;
		long session_mills = session_start.getTime ();

		if (tmp.equals ("PERMANENT")) { // permanent session
			Permanent = true;
		}
		if (session_mills>=now_mills&&session_mills<tomorrow_mills) { // today's session
			Today = true;
		}
		remWaitList (newSess); // firstly remove it from the permanent/waiting list
		if (Permanent||Today) {
			addWaitList (newSess);
		}
	}


	public boolean containsInWaitList (String sessid) {
		boolean flag = false;
		for (int i = 0; i<SessionWaitList.size (); i++) {
			SessionBean tmp = (SessionBean) SessionWaitList.get (i);
			if (sessid.equals (tmp.getSessionid ())) {
				flag = true;
				break;
			}
		}
		return (flag);
	}


	public boolean containsInWaitList (SessionBean sess) {
		String sessid = sess.getSessionid ();
		boolean flag = false;
		for (int i = 0; i<SessionWaitList.size (); i++) {
			SessionBean tmp = (SessionBean) SessionWaitList.get (i);
			if (sessid.equals (tmp.getSessionid ())) {
				flag = true;
				break;
			}
		}
		return (flag);
	}


	public boolean isActive (String sessionID) {
		return (ActiveSessionList.hasSession (sessionID));
	}


	///////////////////////////////////////////////////////////////////////////////
	////// Room Management: updateRoomInfo, removeRoomInfo
	//////////////////////////////////////////////////////////////////////////////
	public boolean updateRoomInfo (RoomBean room) {
		try {
			String commName = room.getCommunityid ();
			CommunityRepository comms = CommunityRepository.getInstance ();
			CommunityBean comm = comms.getCommunity (commName);
			String commType = comm.getCommunitytype ();

			String roomID = room.getRoomid ();
			String sessionID = roomID;

			SessionMediaDescription sessMDes = room.getSessMDes ();
			AdminChannel.UpdateSessionInfo ("NewSession", sessionID, roomID, commName,
									  commType, sessMDes);
			logger.info ("update room info to SS "+sessMDes.toXML ());
			return (true);
		} catch (Exception e) {
			return (false);
		}
	}


	public boolean removeRoomInfo (RoomBean room) {
		try {
			String commName = room.getCommunityid ();
			CommunityRepository comms = CommunityRepository.getInstance ();
			CommunityBean comm = comms.getCommunity (commName);
			String commType = comm.getCommunitytype ();

			String roomID = room.getRoomid ();
			String sessionID = roomID;

			SessionMediaDescription sessMDes = room.getSessMDes ();
			AdminChannel.UpdateSessionInfo ("ByeSession", sessionID, roomID, commName,
									  commType, sessMDes);
			return (true);
		} catch (Exception e) {
			return (false);
		}
	}


	public void checkRoomInfo () {
		AdminChannel.CheckRoomInfo ();
	}


	public void checkRoomInfoReply (QueryRoomInfo reply) {
		LinkedList tmp = reply.getSessionEvents ();
		for (int i = 0; i<tmp.size (); i++) {
			RoomInfoEvent tmpEvt = (RoomInfoEvent) tmp.get (i);
			String commName = tmpEvt.getCommunityName ();
			String roomID = tmpEvt.getRoomID ();
			logger.info ("num "+i+" commnuity name "+commName+" room id "+
					   roomID);
		}
	}


	/**
	 * there is some confusing definition about Session & Room in the session server
	 * (1) XGSP/XGSP/Room ~ related to ActivateSession command
	 * (2) other rooms: like AG/AG/lobby
	 * A session should happen in a room or multiple rooms
	 * right now, a XGSP AV session is created on a XGSP room with the same name as session
	 * and an AG room. for example:
	 * session ID: ourtestroom
	 * session Place: XGSP:ourtestroom;AG:ourtestroom(224.10.10.10)
	 * since the XGSP room name is implicitly the same as the session name, it is not necessary to
	 * define XGSP room in the database. ( Perhas we should it later )
	 * so for each XGSP AV session, we assume there is a XGSP room matching it!!!
	 */
	public boolean updateSessionInfo (SessionBean sess) {
		String sessionID = sess.getSessionid ();
		String commName = "XGSP";
		String commType = "XGSP";
		SessionMediaDescription sessMDes = new SessionMediaDescription ();
		sessMDes.setSessionID (sessionID);

		/// the SessionMediaDesc is useless for XGSP at present, it is just for compatible with AV session
		// define default Audio Des for XGSP room
		MediaDescription AudioDes = new MediaDescription ("AUDIO", "PCMU",
				 "156.56.104.5", 7000);
		MediaDescription VideoDes = new MediaDescription ("VIDEO", "H261",
				 "156.56.104.53", 7000);

		sessMDes.setAudioDesc (AudioDes);
		sessMDes.setVideoDesc (VideoDes);
		AdminChannel.UpdateSessionInfo ("NewSession", sessionID, sessionID, commName,
								  commType, sessMDes);

		return (true);
	}


	public boolean remSessionInfo (SessionBean sess) {
		String sessionID = sess.getSessionid ();
		String commName = "XGSP";
		String commType = "XGSP";
		SessionMediaDescription sessMDes = new SessionMediaDescription ();
		sessMDes.setSessionID (sessionID);

		/// the SessionMediaDesc is useless for XGSP at present, it is just for compatible with AV session
		// define default Audio Des for XGSP room
		MediaDescription AudioDes = new MediaDescription ("AUDIO", "PCMU",
				 "156.56.104.5", 7000);
		MediaDescription VideoDes = new MediaDescription ("VIDEO", "H261",
				 "156.56.104.5", 7000);

		sessMDes.setAudioDesc (AudioDes);
		sessMDes.setVideoDesc (VideoDes);
		AdminChannel.UpdateSessionInfo ("ByeSession", sessionID, sessionID, commName,
								  commType, sessMDes);

		return (true);
	}


	///////////////////////////////////////////////////////////////////////////////
	/// Session Management: ActivateSession, DeActivateSession, ClearAVSession ...
	///////////////////////////////////////////////////////////////////////////////
	public boolean ActivateSession (SessionBean sess) {
		String commid, roomid;
		Vector roomlist;
		RoomRepository rooms;

		try {
			String sess_rooms = sess.getSessionplace ();
			/** ignore the part that sends the command to to other communties **/
			// rooms = RoomRepository.getInstance();
			// Connects all the rooms in the session
			roomlist = ConnectRooms (sess_rooms, true);

			// Send ActiveSession Message to XGSP-SessionServer
			String sess_id = sess.getSessionid ();
			AdminChannel.SetActiveSession (sess_id, true, roomlist);

			// Put the ActiveSession Into the SessionActiveList
			// Maybe we should put it into the ActiveList after we get the reply
			ActiveSession tmp = new ActiveSession (sess);
			ActiveSessionList.addSession (tmp);

		} catch (Exception xx) {
			xx.printStackTrace ();
		}

		return true;
	}


	public boolean DeActivateSession (ActiveSession sess) {
		Vector roomlist;
		String sess_id = sess.get_sessionid ();
		SessionBean sess_info = sess.get_sessioninfo ();
		String sess_rooms = sess_info.getSessionplace ();

		// Disconnects all the rooms
		roomlist = ConnectRooms (sess_rooms, false);
		// Send DeActiveSession Message to XGSP-SessionServer
		//AdminServerChannel admin_channel = AdminServerChannel.getInstance();
		AdminChannel.SetActiveSession (sess_id, false, roomlist); //???

		// Forced all the users that are in the session to logout
		// { memberlist, streamlist, mixed_stream_list }

		// Delete from the Active-Session-List
		ActiveSessionList.remSession (sess_id);

		return false;
	}


	public void clearAVSession () {
		// clear all the active sessions
		ActiveSessionList.clear ();
	}


	public void updateSessionInfo (SessionStatusReport statusRpt) {
		//updateSessionStatus(statusRpt);
	}


	///////////////////////////////////////////////////////////////////////////////////////
	//// Session Stream Management
	//// keeps an A/V stream list for A/V sessions ( because VideoMixer needs that )
	//// (1) addNewAVStream
	//// (2) remAVStream
	//// (3) ActiveToPassive
	//// (4) PassiveToActive
	//////////////////////////////////////////////////////////////////////////////////////
	public void addNewAVStream (StreamBean newAVStream) {
		// get the active session ID
		String sessionID = newAVStream.getSessionID ();
		ActiveSession sess = ActiveSessionList.getSession (sessionID);
		// add this stream to the session
		if (sess!=null) {
			sess.addStream (newAVStream);
		}
	}


	public void remAVStream (StreamBean oldAVStream) {
		// get the active session ID
		String sessionID = oldAVStream.getSessionID ();
		ActiveSession sess = ActiveSessionList.getSession (sessionID);
		// add this stream to the session
		if (sess!=null) {
			sess.remStream (oldAVStream);
		}
	}


	public void ActiveToPassive (StreamBean passiveAV) {
		// get the active session ID
		String sessionID = passiveAV.getSessionID ();
		ActiveSession sess = ActiveSessionList.getSession (sessionID);
		if (sess!=null) {
			sess.disableStream (passiveAV);
		}
	}


	public void PassiveToActive (StreamBean activeAV) {
		// get the active session ID
		String sessionID = activeAV.getSessionID ();
		ActiveSession sess = ActiveSessionList.getSession (sessionID);
		if (sess!=null) {
			sess.enableStream (activeAV);
		}
	}


	////////////////////////////////////////////////////////////////////////////////////////
	//// Session Mixer Management
	//// (1) CreateMixer
	//// (2) DestroyMixer
	//// (3) ModifyMixer
	////////////////////////////////////////////////////////////////////////////////////////
	public String CreateMixer (String sessionID, Vector videoSrc) {
		String mixerID = "vmixer";
		ActiveSession tmpSess = getActiveSession (sessionID);
		int cur_mixer_cnt = tmpSess.GetVideoMixerCnt ()+1;
		mixerID += (new Integer (cur_mixer_cnt)).toString ();
		boolean ok = AdminChannel.SetVideoMixer (videoSrc, sessionID, mixerID, true);
		if (ok) {
			//ActiveSession tmpSess = getActiveSession(sessionID);
			//int cur_mixer_cnt = tmpSess.GetVideoMixerCnt()+ 1 ;
			//mixerID += (new Integer(cur_mixer_cnt)).toString();
			VideoMixer newMixer = new VideoMixer (mixerID);
			newMixer.setSessionID (sessionID);
			ArrayList tmpList = new ArrayList ();
			for (int i = 0; i<videoSrc.size (); i++) {
				StreamBean tmpStrm = (StreamBean) videoSrc.get (i);
				tmpList.add (tmpStrm.getSsrc ());
			}
			newMixer.setMixedStream (tmpList);
			tmpSess.AddVideoMixer (newMixer);
			return (mixerID);
		} else {
			return (null);
		}
	}


	public boolean DestroyMixer (String sessionID, String mixerID) {
		boolean ok = AdminChannel.SetVideoMixer (new Vector (), sessionID, mixerID, false);
		if (ok) {
			ActiveSession tmpSess = getActiveSession (sessionID);
			tmpSess.RemVideoMixer (mixerID);
		}
		return (ok);
	}


	public boolean DestroyMixer (String sessionID, String mixerID, Vector videoSrc) {
		boolean ok = AdminChannel.SetVideoMixer (videoSrc, sessionID, mixerID, false); //???
		return (ok);
	}


	/** modify the exisitng video mixer
	 ** sessionID, mixerID
	 ** NewvideoSrc: streamBean Vector
	 **/
	public boolean ModifyMixer (String sessionID, String mixerID,
						   Vector NewvideoSrc) {
		boolean ok = AdminChannel.SetVideoMixer (NewvideoSrc, sessionID, mixerID, true);
		if (ok) {
			ActiveSession tmpSess = getActiveSession (sessionID);
			VideoMixer mixer = tmpSess.getMixer (mixerID);
			// mixer only need SSRC of the StreamBean
			ArrayList tmpList = new ArrayList ();
			for (int i = 0; i<NewvideoSrc.size (); i++) {
				StreamBean tmpStrm = (StreamBean) NewvideoSrc.get (i);
				tmpList.add (tmpStrm.getSsrc ());
			}
			mixer.setMixedStream (tmpList);
		}
		return (ok);
	}


	// Invoke web services to connect all the rooms in the
	public Vector ConnectRooms (String sess_rooms, boolean op_type) {
		Vector roomlist = new Vector ();
		String commid, roomid;
		String res;

		try {
			java.util.StringTokenizer st = new java.util.StringTokenizer (sess_rooms,
					 ";");
			while (st.hasMoreTokens ()) {
				String tmp = st.nextToken ();
				// tmp format: [ commid:roomid ]
				int split_point = tmp.indexOf (':');
				if (split_point!=-1) {
					commid = tmp.substring (0, split_point);
					roomid = tmp.substring (split_point+1);
				} else {
					commid = "AccessGrid"; // By default : AccessGrid
					roomid = tmp;
				}
				// get roomBean for sending ActiveSession Message
				RoomRepository rooms = RoomRepository.getInstance ();
				RoomBean room = rooms.getRoom (roomid);
				roomlist.add (room);
				/** // Invoke WebServices to connects the rooms
				 CommunityRepository Comms = CommunityRepository.getInstance();
				 CommunityBean community = Comms.getCommunity(commid);
				   //RoomBean room = null;
				   // get the Class of the CommunityStub
				   // service url
				   String comm_type = community.getCommunitytype();
				   if ( comm_type.equals("h323")){
				    String comm_service_url = community.getCommunityaddr();
				    String comm_service_entry = "CommunityServer";
				    String comm_service_class = "";
						  CommunityStub community_service = new CommunityStub( comm_service_url, comm_service_entry, comm_service_class );
				    // get the calling address of the target MCU
				    String xgsp_mcu_addr = xgsp_conf.getH323GW();
				    if ( op_type ) // true --> Make Call
				 res = community_service.MakeCall( xgsp_mcu_addr );
				 else
				 res = community_service.MakeCall( xgsp_mcu_addr ); // ?? Hangup !!
				  System.out.println("web service result: "+res);
				 } **/
			} // end of while
		} catch (Exception xx) {
			xx.printStackTrace ();
		}
		return roomlist;
	}


	/////////////////////////////////////////////////////////////////////////////////////////////////
	//// JoinConference: Add a Memeber to an ActiveSession
	//// LeaveConference: Remove a Memeber from an ActiveSession
	//// SendReplyConfUsers: Send the reply to the NodeManager which requests the list of current users
	////                     in the conference
	////////////////////////////////////////////////////////////////////////////////////////////////
	public void JoinConference (MemberBean newMem, ActiveSession sess) {
		String userID = newMem.getUserid ();
		if (!sess.hasMember (userID)) {
			sess.add_member (newMem);
			ActiveUserList userlist = ActiveUserList.getInstance ();
			UserBean newUser = userlist.getUser (userID);
			AdminChannel.SendJoinConf (sess.get_sessionid (), newUser);
		}
	}


	public void JoinConference (String sessionID, MemberBean newMem) {
		ActiveSession sess = getActiveSession (sessionID);
		String userID = newMem.getUserid ();
		if (sess!=null&&!sess.hasMember (userID)) {
			sess.add_member (newMem);
			ActiveUserList userlist = ActiveUserList.getInstance ();
			UserBean newUser = userlist.getUser (userID);
			AdminChannel.SendJoinConf (sess.get_sessionid (), newUser);
		}
	}


	public void LeaveConference (String sessionID, MemberBean oldMem) {
		ActiveSession old_sess = getActiveSession (sessionID);
		String userID = oldMem.getUserid ();
		if (old_sess!=null) {
			old_sess.rem_member (oldMem);
			ActiveUserList userlist = ActiveUserList.getInstance ();
			UserBean oldUser = userlist.getUser (userID);
			AdminChannel.SendLeaveConf (old_sess.get_sessionid (), oldUser);
		}
	}


	public void LeaveConference (String sessionID, String userID) {
		ActiveSession old_sess = getActiveSession (sessionID);
		if (old_sess!=null) {
			old_sess.rem_member (userID);
			ActiveUserList userlist = ActiveUserList.getInstance ();
			UserBean oldUser = userlist.getUser (userID);
			AdminChannel.SendLeaveConf (old_sess.get_sessionid (), oldUser);
		}
	}


	public void SendReplyConfUsers (String sessionID, String userID) {
		ActiveSession sess = getActiveSession (sessionID);
		MemberBean[] mems = sess.getMembers ();
		ActiveUserList userlist = ActiveUserList.getInstance ();
		ArrayList xgsp_userList = new ArrayList ();
		for (int i = 0; i<mems.length; i++) {
			String tempID = mems[i].getUserid ();
			UserBean tempUser = userlist.getUser (tempID);
			XGSPUser temp = new XGSPUser (tempID, tempUser.getUsername ());
			xgsp_userList.add (temp);
		}
		AdminChannel.SendReplyUserList (sessionID, userID, xgsp_userList);
	}
}