/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.sscomm;

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.logging.*;
import org.jdom.*;

import cgl.narada.jms.*;
import javax.jms.*;

import cgl.globalmmcs.webserver.*;
import cgl.globalmmcs.webserver.sessionmgt.*;


/** this class wrapps the connection between
 ** WebServer and AV-SessionServer
 ** (1) Activate Session
 ** (2) DeActivate Session
 ** (3) SetMixer
 ** (4) Session Status Report From AV Session Server ( every 1 min?? )
 ** (5) StreamEventListener
 ** (6) SessionServer Restarted
 **/
public class AdminServerChannel implements MessageListener {

	private static AdminServerChannel instance;

	private XGSPAVSessionManager manager;

	private String NB_Addr; // NaradaBrokering Address
	private String NB_Port; // NaradaBrokering Port

	private TopicConnectionFactory conFactory;
	private TopicConnection Connection;
	private TopicSession ss_session;
	private Topic ws_ss_topic; // topic between web server and session server
	private TopicPublisher ss_publisher;
	private TopicSubscriber ss_subscriber;

	private String ss_topic_string;
	private String ms_topic_string;

	private String conf_topic0_string, conf_topic1_string;
	private TopicSession conf_session;
	private Topic conf_topic0, conf_topic1;
	private TopicPublisher conf_publisher;
	private TopicSubscriber conf_subscriber;

	private Socket conn_ss;
	private BufferedReader in;
	private PrintWriter out;

	private xml_msg xmlMsg;

	private static Logger logger = Logger.getLogger (AdminServerChannel.class.
			 getName ());
	/** public static AdminServerChannel getInstance()
	  {
	 if ( instance == null ){
	   instance = new AdminServerChannel() ;
	 }
	 return instance;
	  } **/

	public AdminServerChannel () {
		xmlMsg = new xml_msg ();

		XGSPConfiguration xgspconf = XGSPConfiguration.getInstance ();
		this.NB_Addr = xgspconf.getNBAddr ();
		this.NB_Port = xgspconf.getNBPort ();
		// connects with NaradaBroker
		try {
			GesJmsInitializer ini;
			Properties props = new Properties ();
			//props.put("hostname", "elkhart.ucs.indiana.edu"); /** for both tcp, udp*/
			props.put ("hostname", this.NB_Addr);
			props.put ("portnum", this.NB_Port); /** for both tcp, udp */
			props.put ("TCPServerPort", "0"); /** for TCP */
			props.put ("UDPListenerPort", "3046"); /** for UDP */
			props.put ("MulticastHost", "224.224.224.224"); /** for multicast */
			props.put ("MulticastPort", "5045");

			ini = new GesJmsInitializer (props, "tcp");
			conFactory = (TopicConnectionFactory) ini.lookup ();
			Connection = conFactory.createTopicConnection ("guest", "password");
			createNBTopics ();

			Connection.start ();

		} catch (Exception e) {
			e.printStackTrace ();
		}

	}


	public void setManager (XGSPAVSessionManager manager) {
		this.manager = manager;
	}


	private void createNBTopics () throws Exception {
		ss_session = Connection.createTopicSession (false, Session.AUTO_ACKNOWLEDGE);
		ss_topic_string = "/xgsp/av/ss-ws";
		// create publisher/subscriber to the topics
		ws_ss_topic = ss_session.createTopic (ss_topic_string);
		ss_publisher = ss_session.createPublisher (ws_ss_topic);
		ss_subscriber = ss_session.createSubscriber (ws_ss_topic);

		ss_subscriber.setMessageListener (this);

		conf_topic0_string = "/xgsp/conf/ws";
		conf_topic1_string = "/xgsp/conf/client";
		conf_session = Connection.createTopicSession (false,
				 Session.AUTO_ACKNOWLEDGE);
		conf_topic0 = conf_session.createTopic (conf_topic0_string);
		conf_topic1 = conf_session.createTopic (conf_topic1_string);
		conf_publisher = conf_session.createPublisher (conf_topic0);
		conf_subscriber = conf_session.createSubscriber (conf_topic1);

		conf_subscriber.setMessageListener (this);
	}


	public void restart () {
		try {
			Connection.close ();
			Connection = conFactory.createTopicConnection ("guest", "password");
			createNBTopics ();
			Connection.start ();
		} catch (Exception e) {
			e.printStackTrace ();
		}
	}


	///////////////////////////////////////////////////////////////////////////////////
	public synchronized boolean sendMsg (String messageType, String MsgContent) {
		try {
			//xmlMsg.set_xml_msg(MsgContent);
			logger.info ("send message "+messageType+":"+MsgContent);
			//xmlMsg.send_xml_msg(conn_ss.getOutputStream());
			xmlMsg.send_xml_msg (ss_session, ss_publisher, messageType, MsgContent);

			return true;
		} catch (Exception ex) {
			ex.printStackTrace ();
			return false;
		}
	}


	public synchronized void receiveMsg () {
	}


	/**
	 * SetVideoMixer Message
	 */
	public boolean SetVideoMixer (Vector videoSrc, String sessionID,
							String mixerID, boolean active) {
		// send request xml for choose audio src
		XMLGenerator XMLGen = new XMLGenerator ();
		XMLGen.setVideoMixXML (videoSrc, sessionID, mixerID, active);
		String videoMixMsg = XMLGen.getVideoMixXML ();
		//xmlMsg.set_xml_msg(videoMixMsg);
		//System.out.println("Msg to SS for VideoMix"+xmlMsg.get_xml_msg());
		try {
			sendMsg ("SetVideoMixer", videoMixMsg);
			return true;
		} catch (Exception ex) {
			ex.printStackTrace ();
			return false;
		}

	}


	/**
	 * SetActiveSession
	 */
	public boolean SetActiveSession (String sessionid, boolean active,
							   Vector roomlist) {
		// generate xml msg for SeActiveSession
		XMLGenerator XMLGen = new XMLGenerator ();
		XMLGen.setActiveSessionXML (sessionid, active, roomlist);
		String activeSessionMsg = XMLGen.getActiveSessionXML ();
		// send the msg
		try {
			sendMsg ("ActiveSession", activeSessionMsg);
			return true;
		} catch (Exception ex) {
			ex.printStackTrace ();
			return false;
		}
	}


	/**
	 * @para: cmd: NewSession, ByeSession, UnknownAction
	 */
	public boolean UpdateSessionInfo (String cmd, String sessionID, String roomID,
							    String commName, String commType,
							    SessionMediaDescription sessMDes) {
		try {

			UpdateSessionInfo sessionInfo = new UpdateSessionInfo (cmd, commName,
					 commType,
					 roomID, sessionID);
			sessionInfo.setSessionMediaDescription (sessMDes);
			String sessInfoXML = sessionInfo.toXML ("");
			sendMsg ("SessionInformation", sessInfoXML);
			return true;
		} catch (Exception ex) {
			ex.printStackTrace ();
			return false;
		}
	}


	public boolean CheckRoomInfo () {
		QueryRoomInfo request = new QueryRoomInfo ();
		String requestXML = request.toXML ();
		sendMsg ("DefineSessions", requestXML);
		return true;
	}


	//////////////////////////////////////////////////////////////////////////////////////////
	///////   XGSP related messages:
	///////   sendConfMsg, sendReplyUserList, sendJoinConf, sendLeaveConf,
	//////////////////////////////////////////////////////////////////////////////////////////
	public synchronized boolean sendConfMsg (String messageType, String MsgContent) {
		try {
			logger.info ("send conference message "+messageType+":"+MsgContent);
			xmlMsg.send_xml_msg (conf_session, conf_publisher, messageType, MsgContent);
			return true;
		} catch (Exception ex) {
			ex.printStackTrace ();
			return false;
		}
	}


	public boolean SendReplyUserList (String sessionID, String userID,
							    ArrayList userList) {
		ReplyUserList reply = new ReplyUserList ();
		reply.setConfID (sessionID);
		reply.setUserID (userID);
		UserList temp = new UserList ();
		temp.setUserList (userList);
		reply.setUserList (temp);
		String replyXML = reply.toXML ();
		sendConfMsg ("ReplyUserList", replyXML);

		return true;
	}


	public boolean SendJoinConf (String sessionID, UserBean newUser) {
		JoinConf join = new JoinConf ();
		join.setConfID (sessionID);
		XGSPUser temp = new XGSPUser (newUser.getUserid (), newUser.getUsername ());
		join.setNewUser (temp);
		String joinXML = join.toXML ();
		sendConfMsg ("JoinConf", joinXML);

		return true;
	}


	public boolean SendLeaveConf (String sessionID, UserBean oldUser) {
		LeaveConf leave = new LeaveConf ();
		leave.setConfID (sessionID);
		XGSPUser temp = new XGSPUser (oldUser.getUserid (), oldUser.getUsername ());
		leave.setOldUser (temp);
		String leaveXML = leave.toXML ();
		sendConfMsg ("LeaveConf", leaveXML);

		return true;
	}


	public void processRequestUserList (String xml_message) {
		RequestUserList request = new RequestUserList ();
		request.parseClass (xml_message);
		manager.SendReplyConfUsers (request.getConfID (), request.getUserID ());
	}


	// receive the message from the session server
	// (1) SetSession Response
	// (2) SetMixer	  Response
	// (3) StreamEvent
	// (4) Session Status Report
	// (5) Session Server Restarted

	// (6) RequestuserList -- XGSP conference

	// (7)  DefineSessionsReply

	public void onMessage (Message message) {

		if (!(message instanceof TextMessage)) { // we only support TextMessage
			return;
		}

		TextMessage msg = (TextMessage) message;
		try {
			String messageType = (String) msg.getStringProperty ("messagetype");
			logger.info ("web server get msg "+messageType+":"+msg.getText ());

			if (messageType.equalsIgnoreCase ("ActiveSessionReply")) {
				//Vector messages = xmlMsg.receive_xml_msg(msg.getText());

			}
			if (messageType.equalsIgnoreCase ("VideoMixerReply")) {
				//Vector messages = xmlMsg.receive_xml_msg(msg.getText());
			}
			if (messageType.equalsIgnoreCase ("StreamEvent")) {
				//String eventType = msg.getStringProperty("eventtype");

				String xml_message = xmlMsg.receive_xml_msg (msg.getText ());
				StreamBean sBean = getStreamEvent (xml_message);
				String eventType = sBean.getEventType ();
				if (eventType.equalsIgnoreCase ("NewStreamEvent")) {
					manager.addNewAVStream (sBean);
				} else if (eventType.equalsIgnoreCase ("ByeStreamEvent")) {
					manager.remAVStream (sBean);
					// active-to-passive & passive-to-active
				} else if (eventType.equalsIgnoreCase ("ActiveToPassive")) {
					manager.ActiveToPassive (sBean);
				} else if (eventType.equalsIgnoreCase ("PassiveToActive")) {
					manager.PassiveToActive (sBean);

				}
			}
			if (messageType.equalsIgnoreCase ("SessionStatusReport")) {
				String xml_message = xmlMsg.receive_xml_msg (msg.getText ());
				SessionStatusReport rpt = getSessionReport (xml_message);
				logger.fine (rpt.toXML (""));
				//manager.updateSessionInfo(rpt);
			}
			if (messageType.equalsIgnoreCase ("SSSRestarted")) {
				String xml_message = xmlMsg.receive_xml_msg (msg.getText ());
				manager.clearAVSession ();
			}
			if (messageType.equalsIgnoreCase ("DefineSessionsReply")) {
				String xml_message = xmlMsg.receive_xml_msg (msg.getText ());
				logger.info ("get define  Sessions Reply");
				processDefineSessionsReply (xml_message);
			}
			// conference Topic messages
			if (messageType.equalsIgnoreCase ("RequestuserList")) {
				String xml_message = xmlMsg.receive_xml_msg (msg.getText ());
				processRequestUserList (xml_message);
			}
		} catch (Exception e) {
			//LogManager.err("Textchat","***** Unable to extract  the message *****",e);
			e.printStackTrace ();
		}
	}


	/** <SetActiveSessionReply>
	 <SessionID>  </SessionID>	<Result >  </Result > <Reason>  </Reason>
	 **/
	public void getActiveSessionReply (String inMsg) {
		//String inMsg = (String)messages.elementAt(0);
		SessionParser sp = new SessionParser ();
		sp.parse (inMsg);
		Element eRoot = sp.getRootElement ();
		Iterator it = eRoot.getChildren ().iterator ();
		//if (it.hasNext()) {

		//}

	}


	/**
	 ** <SetVideoMixerReply> <SessionID>  </SessionID> <Result >  </Result > <Reason>  </Reason> </SetVideoMixerReply>
	 **/
	public void getVideoMixerReply (String inMsg) {
		//String inMsg = (String)messages.elementAt(0);
		SessionParser sp = new SessionParser ();
		sp.parse (inMsg);
		Element eRoot = sp.getRootElement ();
		Iterator it = eRoot.getChildren ().iterator ();
		if (it.hasNext ()) {

		}
	}


	/**
	 **/
	public StreamBean getStreamEvent (String inMsg) {
		//String inMsg = (String)messages.elementAt(0);
		SessionParser sp = new SessionParser ();
		sp.parse (inMsg);
		Element eRoot = sp.getRootElement ();

		StreamBean sBean = new StreamBean ();

		sBean.setStreamBean (eRoot);

		// sBean.setStreamBean( eRoot, "video");

		return (sBean);
	}


	/** <SessionID> xxx </SessionID>
	    <SessionStatusInfo>
	    <CurParticipant> </CurParticipant> // number
	    <PastParticipant> </PastParticipant>
	    </SessionStatusInfo>
	 **/
	public SessionStatusReport getSessionReport (String inMsg) {
		String sessionID;
		String cur_participants;
		String past_participants;

		//String inMsg = (String)messages.elementAt(0);
		SessionParser sp = new SessionParser ();
		sp.parse (inMsg);
		Element eRoot = sp.getRootElement ();

		SessionStatusReport report = new SessionStatusReport ();
		report.parseClass (eRoot);
		return (report);
	}


	/// <SSRestarted>  <Time> xxx </Time>    <Reason> xxx </Reason> </SSRestarted>
	public void getSSRestarted (String inMsg) {

	}


	///
	public void processDefineSessionsReply (String xml_message) {
		QueryRoomInfo reply = new QueryRoomInfo ();
		reply.parseClass (xml_message);
		// manager how to handle ??
		manager.checkRoomInfoReply (reply);
	}


	public static void main (String[] args) {
		AdminServerChannel adminChan = new AdminServerChannel ();
		XGSPAVSessionManager manager = XGSPAVSessionManager.getInstance ();
		adminChan.setManager (manager);

		Vector roomlist = new Vector ();
		RoomBean tmp = new RoomBean ();
		tmp.setRoomid ("lobby");
		tmp.setCommunityid ("AccessGrid");
		tmp.setRoomtype ("AG");
		roomlist.add (tmp);
		adminChan.SetActiveSession ("testsession", true, roomlist);

	}
}
