/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.sscomm;

import org.apache.xerces.parsers.DOMParser;
import org.jdom.*;
import java.util.*;


/**
 * <p>Title: </p>
 * <p>Description: XGSP Conference Web Server</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Community Grids Lab, Indiana University</p>
 * @author Wenjun Wu
 * @version 1.0
 */

public class QueryRoomInfo extends XMLMessage {
	private LinkedList sessionEvents = new LinkedList ();
	private boolean request = false;
	private boolean response = true;

	public QueryRoomInfo () {
		this.msgType = "DefineSessions";
	}


	public Object parseClass (Element e) {
		this.msgType = "DefineSessions";
		sessionEvents.clear ();
		try {
			Iterator it = e.getChildren ().iterator ();
			while (it.hasNext ()) {
				e = (Element) it.next ();
				if (e.getName ().equalsIgnoreCase ("SessionInformationEvent")) {
					RoomInfoEvent sie = new RoomInfoEvent ();
					sie.parseClass (e);
					sessionEvents.add (sie);
				}
			}
			if (sessionEvents.size ()<=0) {
				request = true;
				response = false;
			}
		} catch (Exception ex) {
		}
		return this;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param ds DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public Object cloneClass (XMLMessage ds) {
		this.msgType = "DefineSessions";
		sessionEvents.clear ();
		sessionEvents = ((QueryRoomInfo) ds).getSessionEvents ();
		if (sessionEvents.size ()<=0) {
			request = true;
			response = false;
		}
		return this;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param request DOCUMENT ME!
	 */
	public void setRequest (boolean request) {
		this.request = request;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean isRequest () {
		return request;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param response DOCUMENT ME!
	 */
	public void setResponse (boolean response) {
		this.response = response;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean isResponse () {
		return response;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getSample () {
		String sample = null;
		sample = "<DefineSessions>";
		sample += "</DefineSessions>";
		return sample;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getSampleReply () {
		String sample = null;
		RoomInfoEvent sie = new RoomInfoEvent ();
		sample = "<DefineSessionsReply>";
		sample += sie.getSample ();
		sample += sie.getSample ();
		sample += sie.getSample ();
		sample += "</DefineSessionsReply>";
		return sample;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param sessionEvents DOCUMENT ME!
	 */
	public void setSessionEvents (LinkedList sessionEvents) {
		this.sessionEvents = sessionEvents;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public LinkedList getSessionEvents () {
		return sessionEvents;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param delimiter DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String toXML (String delimiter) {
		String definedSessions = "";
		definedSessions += "<DefineSessions>";
		definedSessions += delimiter;
		definedSessions += "</DefineSessions>";
		return definedSessions;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param delimiter DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String toXMLReply (String delimiter) {
		String definedSessions = "";
		definedSessions += "<DefineSessionsReply>";
		definedSessions += delimiter;
		for (int i = 0; i<sessionEvents.size (); i++) {
			definedSessions +=
					 ((RoomInfoEvent) sessionEvents.get (i)).toXML (delimiter+"  ");
			definedSessions += delimiter;
		}
		definedSessions += "</DefineSessionsReply>";
		return definedSessions;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String toXMLReply () {
		String definedSessions = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		definedSessions += toXMLReply ("");
		return definedSessions;
	}

}