/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.sscomm;

import org.apache.xerces.parsers.DOMParser;
import org.jdom.*;
import java.util.*;


public class RoomInfoEvent extends XMLMessage {
	//~ Instance fields ----------------------------------------------------------

	private String communityName = "";
	private String communityType = "";
	private String eventType = "UnknownEvent"; // NewSessionEvent, ByeSessionEvent, AvailableSessionEvent, UnknownEvent
	private String roomID = "";
	private String sessionID = "";

	//~ Constructors -------------------------------------------------------------

	/**
	 * Generates a new RoomInfoEvent object.
	 */
	public RoomInfoEvent () {
		this.msgType = "SessionInformationEvent";
	}


	//~ Methods ------------------------------------------------------------------

	/**
	 * DOCUMENT ME!
	 *
	 * @param e DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public Object parseClass (Element e) {
		this.msgType = "SessionInformationEvent";
		try {
			Iterator it = e.getChildren ().iterator ();
			while (it.hasNext ()) {
				e = (Element) it.next ();
				if (e.getName ().equalsIgnoreCase ("CommunityType")) {
					communityType = e.getTextTrim ();
				} else if (e.getName ().equalsIgnoreCase ("CommunityName")) {
					communityName = e.getTextTrim ();
				} else if (e.getName ().equalsIgnoreCase ("RoomID")) {
					roomID = e.getTextTrim ();
				} else if (e.getName ().equalsIgnoreCase ("SessionID")) {
					sessionID = e.getTextTrim ();
				} else if (e.getName ().equalsIgnoreCase ("EventType")) {
					eventType = e.getTextTrim ();
					if (!(eventType.equalsIgnoreCase ("NewSessionEvent"))||
					    !(eventType.equalsIgnoreCase ("ByeSessionEvent"))||
					    !(eventType.equalsIgnoreCase ("AvailableSessionEvent"))) {
						eventType = "UnknownEvent";
					}
				}
			}
		} catch (Exception ex) {
		}
		return this;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param si DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public Object cloneClass (XMLMessage si) {
		this.msgType = "SessionInformationEvent";
		communityType = ((RoomInfoEvent) si).getCommunityType ();
		communityName = ((RoomInfoEvent) si).getCommunityName ();
		roomID = ((RoomInfoEvent) si).getRoomID ();
		sessionID = ((RoomInfoEvent) si).getSessionID ();
		eventType = ((RoomInfoEvent) si).getEventType ();
		return this;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param communityName DOCUMENT ME!
	 */
	public void setCommunityName (String communityName) {
		this.communityName = communityName;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getCommunityName () {
		return communityName;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param communityType DOCUMENT ME!
	 */
	public void setCommunityType (String communityType) {
		this.communityType = communityType;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getCommunityType () {
		return communityType;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param eventType DOCUMENT ME!
	 */
	public void setEventType (String eventType) {
		if (eventType.equalsIgnoreCase ("NewSessionEvent")||
		    eventType.equalsIgnoreCase ("ByeSessionEvent")||
		    eventType.equalsIgnoreCase ("AvailableSessionEvent")) {
			this.eventType = eventType;
		} else {
			this.eventType = "UnknownEvent";
		}
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getEventType () {
		return eventType;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param roomID DOCUMENT ME!
	 */
	public void setRoomID (String roomID) {
		this.roomID = roomID;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getRoomID () {
		return roomID;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getSample () {
		String sample = null;
		sample = "<SessionInformationEvent>";
		sample += "<CommunityType>XGSP</CommunityType>";
		sample += "<CommunityName>XGSP</CommunityName>";
		sample += "<RoomID>lobby</RoomID>";
		sample += "<SessionID>lobby</SessionID>";
		sample += "<EventType>NewSessionEvent</EventType>";
		sample += "</SessionInformationEvent>";
		return sample;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param sessionID DOCUMENT ME!
	 */
	public void setSessionID (String sessionID) {
		this.sessionID = sessionID;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String getSessionID () {
		return sessionID;
	}


	/**
	 * DOCUMENT ME!
	 *
	 * @param delimiter DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public String toXML (String delimiter) {
		String sessioninformation = "";
		sessioninformation += "<SessionInformationEvent>";
		sessioninformation += delimiter;
		sessioninformation += ("  <CommunityType>"+communityType+
						   "</CommunityType>");
		sessioninformation += delimiter;
		sessioninformation += ("  <CommunityName>"+communityName+
						   "</CommunityName>");
		sessioninformation += delimiter;
		sessioninformation += ("  <RoomID>"+roomID+"</RoomID>");
		sessioninformation += delimiter;
		sessioninformation += ("  <SessionID>"+sessionID+"</SessionID>");
		sessioninformation += delimiter;
		sessioninformation += ("  <EventType>"+eventType+"</EventType>");
		sessioninformation += delimiter;
		sessioninformation += "</SessionInformationEvent>";
		return sessioninformation;
	}
}
