/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.sscomm;

import java.util.*;
import java.io.*;
import java.net.*;

import org.jdom.*;
import org.jdom.input.*;
import org.jdom.output.*;
import org.jdom.output.XMLOutputter;
import org.jdom.input.SAXBuilder;
import org.xml.sax.SAXException;


public class SessionParser {

	private Document doc = null;
	private String xmlString = null;
	private File xmlFile = null;
	private String xmlHeader = null;
//    private static String sp= "";
	private boolean isElementFound = false;
	private boolean isAttributeFound = false;

	//Document docHeader = null;

	public SessionParser () {}


	public String getRootElementName () {
		Element e = this.doc.getRootElement ();
		return e.getName ();
	}


	public Element getRootElement () {
		Element e = this.doc.getRootElement ();
		return e;
	}


	/**
	 * @return: false ~ parse fail; true ~ parse succeed
	 */
	public boolean parse (String xmlString) {
		try {
			int index = xmlString.indexOf ("<?xml");
			if (index!=-1) {
				xmlString = xmlString.substring (index, xmlString.length ());
			}
			this.xmlString = xmlString;
			StringReader sr = new StringReader (xmlString);
			SAXBuilder builder = new SAXBuilder ();
			this.doc = builder.build (sr);

			return (true);
		} catch (JDOMException e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
			return (false);
		}
	}


	public void parse (String xmlString, String xmlHeader) {
		try {
			int index = xmlString.indexOf ("<?xml");
			if (index!=-1) {
				xmlString = xmlString.substring (index, xmlString.length ());
			}
			this.xmlString = xmlString;
			this.xmlHeader = xmlHeader;
			StringReader sr = new StringReader (xmlString);
			SAXBuilder builder = new SAXBuilder ();
			this.doc = builder.build (sr);
		} catch (JDOMException e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
	}


	public void parse (File xmlFile) {
		try {
			this.xmlFile = xmlFile;
			SAXBuilder builder = new SAXBuilder ();
			this.doc = builder.build (xmlFile);
		} catch (JDOMException e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
	}


	public String getElementValue (String elementName) {
		String elementValue = null;
		isElementFound = false;
		try {
			Iterator i = this.doc.getRootElement ().getChildren ().iterator ();
			//System.out.println("Element Name root "+elementName);
			elementValue = this.getElementValueTail (i, elementName);
		} catch (Exception e) {
		}
		return elementValue;
	}


	public String getElementValueTail (Iterator i, String elementName) {
		String elementValue = null;
		try {
			while (i.hasNext ()) {
				Element e = (Element) i.next ();
				if (e.getName ().equalsIgnoreCase (elementName)) {
					//System.out.println(elementName+" "+i+" Element value "+e.getTextTrim());
					elementValue = e.getTextTrim ();
					isElementFound = true;
					break;
				} else {
					Iterator ii = e.getChildren ().iterator ();
					elementValue = this.getElementValueTail (ii, elementName);
					if (isElementFound) {
						break;
					}
				}
			}
		} catch (Exception e) {
		}
		return elementValue;
	}


	public String getAttributeValue (String elementName, String AttributeName) {
		String attributeValue = null;
		isAttributeFound = false;
		try {
			Iterator i = this.doc.getRootElement ().getChildren ().iterator ();
			attributeValue = this.getAttributeValueTail (i, elementName, AttributeName);
		} catch (Exception e) {
		}
		return attributeValue;
	}


	public String getAttributeValueTail (Iterator i, String elementName,
								  String AttributeName) {
		String attributeValue = null;
		try {
			while (i.hasNext ()) {
				Element e = (Element) i.next ();
				if (e.getName ().equalsIgnoreCase (elementName)) {
					attributeValue = e.getAttributeValue (AttributeName);
					isAttributeFound = true;
					break;
				} else {
					Iterator ii = e.getChildren ().iterator ();
					attributeValue = this.getAttributeValueTail (ii, elementName,
							 AttributeName);
					if (isAttributeFound) {
						break;
					}
				}
			}
		} catch (Exception e) {
		}
		return attributeValue;
	}


	/**
	 *
	 * @param elementName
	 * @param elementValue
	 */

	public void setElementValue (String elementName, String elementValue) {
		try {
			Iterator i = this.doc.getRootElement ().getChildren ().iterator ();
			setElementValueTail (i, elementName, elementValue);
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
	}


	/**
	 *
	 * @param elementName
	 * @param elementValue
	 */
	public void setElementValueTail (Iterator i, String elementName,
							   String elementValue) {
		try {
			while (i.hasNext ()) {
				Element e = (Element) i.next ();
				//System.out.println("Root element name is "+e.getName()+" = "+e.getTextTrim());
				if (e.getName ().equalsIgnoreCase (elementName)) {
					e.setText (elementValue);
				} else {
					Iterator ii = e.getChildren ().iterator ();
					setElementValueTail (ii, elementName, elementValue);
				}
			}
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
	}


	/**
	 *
	 * @param elementName
	 * @param attributeName
	 * @param attributeValue
	 */
	public void setAtributeValue (String elementName, String attributeName,
							String attributeValue) {
		try {
			Iterator i = this.doc.getRootElement ().getChildren ().iterator ();
			setAtributeTail (i, elementName, attributeName, attributeValue);
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
	}


	/**
	 *
	 * @param i
	 * @param elementName
	 * @param attributeName
	 * @param attributeValue
	 */
	public void setAtributeTail (Iterator i, String elementName,
						    String attributeName, String attributeValue) {
		try {
			while (i.hasNext ()) {
				Element ee = (Element) i.next ();
				//System.out.println("   Child element name is "+ee.getName()+" = "+ee.getTextTrim());
				if (ee.getName ().equalsIgnoreCase (elementName)) {
					ee.removeAttribute (attributeName);
					ee.addAttribute (attributeName, attributeValue);
				} else {
					Iterator ii = ee.getChildren ().iterator ();
					setAtributeTail (ii, elementName, attributeName, attributeValue);
				}
			}
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
	}


	/**
	 *
	 * @param elementName
	 * @param attributeName
	 * @param attributeValue
	 */

	public void addAtribute (String elementName, String attributeName,
						String attributeValue) {
		try {
			Iterator i = this.doc.getRootElement ().getChildren ().iterator ();
			addAtributeTail (i, elementName, attributeName, attributeValue);
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
	}


	public void addAtributeTail (Iterator i, String elementName,
						    String attributeName, String attributeValue) {
		try {
			while (i.hasNext ()) {
				Element ee = (Element) i.next ();
				//System.out.println("   Child element name is "+ee.getName()+" = "+ee.getTextTrim());
				if (ee.getName ().equalsIgnoreCase (elementName)) {
					ee.removeAttribute (attributeName);
					ee.addAttribute (attributeName, attributeValue);
				} else {
					Iterator ii = ee.getChildren ().iterator ();
					addAtributeTail (ii, elementName, attributeName, attributeValue);
				}
			}
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
	}


	public void addElement (String parentElementName, String elementName,
					    String elementValue) {
		try {
			if (parentElementName==null||parentElementName.length ()==0) {
				Element ee = new Element (elementName);
				ee.setText (elementValue);
				this.doc.addContent (ee);
			} else {
				Iterator i = this.doc.getRootElement ().getChildren ().iterator ();
				addElementTail (i, parentElementName, elementName, elementValue);
			}
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
	}


	/**
	 *
	 * @param i
	 * @param parentElementName
	 * @param elementName
	 * @param elementValue
	 */

	public void addElementTail (Iterator i, String parentElementName,
						   String elementName, String elementValue) {
		try {
			while (i.hasNext ()) {
				Element ee = (Element) i.next ();
				//System.out.println("   Child element name is "+ee.getName()+" = "+ee.getTextTrim());
				if (ee.getName ().equalsIgnoreCase (parentElementName)) {
					Element new_element = new Element (elementName);
					new_element.setText (elementValue);
					ee.removeChildren ();
					//.removeChild(elementName);
					//System.out.print("Hello there\n"+this.isElementExist(elementName));
					//System.out.print(this.getXMLDocument());
					ee.addContent (new_element);
					//System.out.print(this.getXMLDocument());
				} else {
					Iterator ii = ee.getChildren ().iterator ();
					addElementTail (ii, parentElementName, elementName, elementValue);
				}
			}
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
	}


	/**
	 *
	 * @return
	 */

	public String getXMLDocument () {
		try {
			XMLOutputter xmlOutputter = new XMLOutputter ();
			return xmlOutputter.outputString (this.doc);
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
		return "";
	}


	/**
	 *
	 * @param elementName
	 * @return
	 */

	public boolean isElementExist (String elementName) {
		boolean isExist = false;
		try {
			Iterator i = this.doc.getRootElement ().getChildren ().iterator ();
			isExist = isElementExistTail (i, elementName);
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
		return isExist;
	}


	public boolean isElementExistTail (Iterator i, String elementName) {
		boolean isExist = false;
		try {
			while (i.hasNext ()) {
				Element ee = (Element) i.next ();
				//System.out.println("   Child element name is "+ee.getName()+" = "+ee.getTextTrim());
				if (ee.getName ().equalsIgnoreCase (elementName)) {
					return true;
				} else {
					Iterator ii = ee.getChildren ().iterator ();
					isExist = isElementExistTail (ii, elementName);
				}
			}
		} catch (Exception e) {
			System.out.println ("There is ERROR \n"+e.getMessage ());
		}
		return isExist;
	}
}