/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.sscomm;

import java.io.*;
import java.net.*;
import java.util.*;
import org.jdom.*;

import cgl.narada.jms.*;
import javax.jms.*;

import cgl.globalmmcs.webserver.*;


/**
 * Unit Test classes for XML message
 */
public class TestXMLComm implements MessageListener {
	private TopicSession ss_session;
	private Topic ws_ss_topic; // topic between web server and session server
	private TopicPublisher ss_publisher;
	private TopicSubscriber ss_subscriber;

	private String ss_topic_string;

	public TestXMLComm () {
		try {
			GesJmsInitializer ini;
			Properties props = new Properties ();
			props.put ("hostname", "murray.ucs.indiana.edu"); /** for both tcp, udp*/
			props.put ("portnum", "5045"); /** for both tcp, udp */
			props.put ("TCPServerPort", "0"); /** for TCP */
			props.put ("UDPListenerPort", "3046"); /** for UDP */
			props.put ("MulticastHost", "224.224.224.224"); /** for multicast */
			props.put ("MulticastPort", "5045");
			ini = new GesJmsInitializer (props, "tcp");
			TopicConnectionFactory conFactory = (TopicConnectionFactory) ini.lookup ();
			TopicConnection Connection = conFactory.createTopicConnection ("guest",
					 "password");

			ss_session = Connection.createTopicSession (false,
					 Session.AUTO_ACKNOWLEDGE);

			ss_topic_string = "/xgsp/av/ss-ws";

			//ms_topic_string = "/globalmmcs/avctrl/";
			// create publisher/subscriber to the topics
			ws_ss_topic = ss_session.createTopic (ss_topic_string);
			ss_publisher = ss_session.createPublisher (ws_ss_topic);
			ss_subscriber = ss_session.createSubscriber (ws_ss_topic);

			ss_subscriber.setMessageListener (this);
			Connection.start ();

		} catch (Exception e) {
			System.out.println ("hate");
		}
	}


	public void onMessage (Message message) {
		if (!(message instanceof TextMessage)) { // we only support TextMessage
			return;
		}

		TextMessage msg = (TextMessage) message;
		try {
			String messageType = (String) msg.getStringProperty ("messagetype");
			if (messageType.equalsIgnoreCase ("ActiveSession")) {
				String textMsg = msg.getText ();
				System.out.println (textMsg);
				// send the reply
				// send the event
				//Thread.sleep( 30*1000);
				ArrayList strms = PrepareStrmEvt ();
				for (int i = 0; i<strms.size (); i++) {
					StreamBean tmp = (StreamBean) strms.get (i);
					sendStreamEvent (tmp);
				}
				sendSessionRpt ();
			}
			if (messageType.equalsIgnoreCase ("VideoMixer")) {
				String textMsg = msg.getText ();
				System.out.println (textMsg);
			}

		} catch (Exception e) {
			//LogManager.err("Textchat","***** Unable to extract  the message *****",e);
			e.printStackTrace ();
		}
	}


	public ArrayList PrepareStrmEvt () {
		ArrayList allStrm = new ArrayList ();

		StreamBean tmp = new StreamBean ();
		tmp.setStreamtype (0); //0: video; 1: audio
		tmp.setUsername ("wwj");
		tmp.setSsrc ("12345");
		tmp.setStrdescription ("xxxx");
		tmp.setImgpath ("/xxx/xxx");
		tmp.setSessionID ("ourtestroom");
		allStrm.add (tmp);

		tmp = new StreamBean ();
		tmp.setStreamtype (0); //0: video; 1: audio
		tmp.setUsername ("qqq");
		tmp.setSsrc ("78910");
		tmp.setStrdescription ("xxxx");
		tmp.setImgpath ("/xxx/xxx");
		tmp.setSessionID ("ourtestroom");
		allStrm.add (tmp);

		tmp = new StreamBean ();
		tmp.setStreamtype (0); //0: video; 1: audio
		tmp.setUsername ("jjj");
		tmp.setSsrc ("88888");
		tmp.setStrdescription ("xxxx");
		tmp.setImgpath ("/xxx/xxx");
		tmp.setSessionID ("ourtestroom");
		allStrm.add (tmp);

		return (allStrm);
	}


	public void sendStreamEvent (StreamBean stream) {
		String xmlMsg = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		String event_type = "NewStreamEvent";

		xmlMsg += "<StreamEvent>";
		xmlMsg += "<EventType>"+event_type+"</EventType>";
		xmlMsg += "<SessionID>"+stream.getSessionID ()+"</SessionID>";
		xmlMsg += "<MediaType>"+stream.getStreamtype ()+"</MediaType>";
		xmlMsg += "<Stream>";
		xmlMsg += "<UserName>"+stream.getUsername ()+"</UserName>";
		xmlMsg += "<SSRC>"+stream.getSsrc ()+"</SSRC>";
		xmlMsg += "<Description>"+stream.getStrdescription ()+"</Description>";
		xmlMsg += "</Stream>";
		xmlMsg += "<ImagePath>"+stream.getImgpath ()+"</ImagePath>";
		xmlMsg += "</StreamEvent>";

		try {
			TextMessage msg = (TextMessage) ss_session.createTextMessage ();
			msg.setStringProperty ("messagetype", "StreamEvent");
			//msg.setStringProperty("eventtype", "NewStreamEvent" );
			msg.setText (xmlMsg);
			ss_publisher.publish (msg);
		} catch (Exception e) {
			e.printStackTrace ();
		}
	}


	public void sendSessionRpt () {
		String xmlMsg = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
		xmlMsg +=
				 "<SessionStatusReport>  <SessionReport>    <SessionID>ourtestroom</SessionID>";
		xmlMsg += "<SessionStatusInfo>   <CurParticipant>0</CurParticipant>   <PastParticipant>0</PastParticipant>    </SessionStatusInfo>  </SessionReport>";
		xmlMsg += "</SessionStatusReport>";
		try {
			TextMessage msg = (TextMessage) ss_session.createTextMessage ();
			msg.setStringProperty ("messagetype", "SessionStatusReport");
			//msg.setStringProperty("eventtype", "NewStreamEvent" );
			msg.setText (xmlMsg);
			ss_publisher.publish (msg);
		} catch (Exception e) {
			e.printStackTrace ();
		}
	}


	public static void main (String[] args) {
		TestXMLComm test = new TestXMLComm ();
	}

}