/**
 * Software License, Version 1.0
 *
 * Copyright 2003 The Trustees of Indiana University.  All rights reserved.
 *
 *
 *Redistribution and use in source and binary forms, with or without
 *modification, are permitted provided that the following conditions are met:
 *
 *1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 *2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 *3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 *
 *"This product includes software developed by the Community Grids Lab. For
 * further information contact the Community Grids Lab at
 * http://communitygrids.iu.edu/."
 *
 * Alternatively, this acknowledgement may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 *
 *4) The name Indiana University or Community Grids Lab or GlobalMMCS,
 * shall not be used to endorse or promote products derived from this software
 * without prior written permission from Indiana University.  For written
 * permission, please contact the Advanced Research and Technology Institute
 * ("ARTI") at 351 West 10th Street, Indianapolis, Indiana 46202.
 *5) Products derived from this software may not be called GlobalMMCS,
 * nor may Indiana University or Community Grids Lab or GlobalMMCS appear
 * in their name, without prior written permission of ARTI.
 *
 *
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of
 * any other entity.  Indiana University disclaims any liability to any
 * recipient for claims brought by any other entity based on infringement of
 * intellectual property rights or otherwise.
 *
 *LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 *WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 *NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 *INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 *INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 *"VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 *LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 *ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 *GENERATED USING SOFTWARE.
 */
package cgl.globalmmcs.webserver.user;

/**
 * <p>Title: </p>
 * <p>Description: XGSP Conference Web Server</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Community Grids Lab, Indiana University</p>
 * @author Wenjun Wu
 * @version 1.0
 */

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

import cgl.globalmmcs.webserver.*;
import cgl.globalmmcs.webserver.sessionmgt.XGSPAVSessionManager;
import cgl.globalmmcs.webserver.sessionmgt.ActiveSession;
import cgl.globalmmcs.webserver.sscomm.AdminServerChannel;


/**
 * this servelet commands guide the user into the active XGSP conference
 * it keeps some old code which helps to implement session refreshment in web
 * server.
 * The code may be useful for support the low bandwidth user who can access the web pages
 * of the conference. so we keep the code for future.
 */
public class joinSession implements Command {
	private String next;

	public joinSession (String next) { // joinSession->enterRoom
		this.next = next;
	}


	public String execute (HttpServletRequest req) throws CommandException {
		String commid, roomid;
		try {
			// get the parameter for new sessionID
			String sess_id = req.getParameter ("sessionid");
			ActiveSession sess;

			// Three cases:
			// (1) The user hasn't join in any session: create new member and add this member into the activeSession
			// (2) The user has joined in the same session as this one, and do nothing
			// (3) The user has joined in the different session from this one, then force the user quit that session
			//     and join this session

			// Get the user info from the http-session
			HttpSession html_sess = req.getSession (false); // html_sess may be null, pay attention!!
			UserBean user = (UserBean) html_sess.getValue ("user");

			MemberBean oldmem = (MemberBean) html_sess.getValue ("member");
			XGSPAVSessionManager sessMgr = XGSPAVSessionManager.getInstance ();

			if (oldmem!=null&&oldmem.getSessionid ()!=sess_id) { // case(3)
				// quit the old session
				String old_sess_id = oldmem.getSessionid ();
				sessMgr.LeaveConference (old_sess_id, oldmem);
				// if necessary, even remove old session
			}
			// if case(2) then skip the following section
			// create the new mem, try to check whether this user has joined in another session
			// if that delete him from that session ( !! )
			MemberBean mem = new MemberBean (); // create the new member
			mem.setUserid (user.getUserid ());
			mem.setSessionid (sess_id);
			html_sess.putValue ("member", mem); // update the value in html-session

			// update active-session-list
			sessMgr.JoinConference (sess_id, mem);
			//start up stream-monitor thread ( been removed!! )

			// Set the Attributes of the membership of this session
			sess = sessMgr.getActiveSession (sess_id);
			req.setAttribute ("members", sess.getMembers ());
			req.setAttribute ("sessionid", sess_id);
			req.setAttribute ("session_info", sess.get_sessioninfo ());
			req.setAttribute ("user", user);
			// EnterRoom.jsp will show the active memebers
			return (next);

		} catch (Exception fe) {
			throw new CommandException ("Join Session "+fe.getMessage ());
		}

	}
}