package progress.message.jclient;

import cgl.narada.jms.*;
import javax.jms.*;
import java.util.*;

/** 
    The purpose of this class is to make switching between SonicMQ and 
    NaradaBrokering transparent to the application. All the sonicMQ Jms client
    has to do is to remove the sonic.jar files from the classpath.
    Thus there would be two seperate environment variables, one for Sonic Jar
    files and one for NaradaBrokering Jar files. Depending on which provider
    you need to use simply add the appropriate environment variable to the
    classpath variable.

    @author Shrideep Pallickara
    $Date$
    $Revision$
*/

public class TopicConnectionFactory 
  implements javax.jms.TopicConnectionFactory {

  boolean connFactoryInit = false;
  GesJmsInitializer ini;
  javax.jms.TopicConnectionFactory connFactory;

  public TopicConnectionFactory(String brokerInfo) {
    StringTokenizer tok = new StringTokenizer(brokerInfo, ":");
    if (tok.countTokens() == 2) {
      String hostName = tok.nextToken().trim();
      int portNum = Integer.parseInt(tok.nextToken().trim());
      //ini = new GesJmsInitializer(hostName, portNum);
      ini = new GesJmsInitializer(hostName, portNum, "niotcp");
      connFactory = (javax.jms.TopicConnectionFactory)ini.lookup();
    }

  }

  public javax.jms.TopicConnection 
  createTopicConnection() throws JMSException {
    if (connFactory==null) {
      throw new javax.jms.JMSException("The Connection Factory was not " + 
				       "initialized, check <host:port>");
    }
    return connFactory.createTopicConnection();
  }
  
  public javax.jms.TopicConnection 
  createTopicConnection(String userName, String passwd) throws JMSException {
    if (connFactory==null) {
      throw new javax.jms.JMSException("The Connection Factory was not " + 
				       "initialized, check <host:port>");
    }
    return connFactory.createTopicConnection(userName, passwd);
  }


  public static void main(String[] args) {
    if (args.length != 1) {
      System.out.println("Usage java progress.message.jclient" +
			 ".TopicConnectionFactory" + "brokerHost:brokerPort");
    }
    
    TopicConnectionFactory factory = new TopicConnectionFactory(args[0]);
    try {
      factory.createTopicConnection();
    } catch (JMSException jmsEx) {
      System.out.println(jmsEx);
    }
  }

}


